from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("clk125_gtp", 0,
        Subsignal("p", Pins("D6")),
        Subsignal("n", Pins("D5")),
    ),

    ("serial", 0,
        Subsignal("tx", Pins("U17")),
        Subsignal("rx", Pins("T17")),
        IOStandard("LVCMOS33")
    ),

    # HMC clocking chips (830 and 7043)
    ("hmc830_pwr_en", 0, Pins("V7"), IOStandard("LVCMOS25")),
    ("hmc7043_out_en", 0, Pins("V8"), IOStandard("LVCMOS25")),
    ("hmc_spi", 0,
        Subsignal("clk", Pins("T18"), Misc("PULLDOWN=TRUE")),
        # cs[0]=830 cs[1]=7043
        # Watch out for the HMC830 SPI mode peculiarity. PULLDOWN CS here
        # so that toggling the SPI core offline will make edges.
        Subsignal("cs_n", Pins("K16 R17"), Misc("PULLDOWN=TRUE")),
        Subsignal("mosi", Pins("R18"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("J15"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS33")
    ),
    ("hmc7043_reset", 0, Pins("J18"), IOStandard("LVCMOS33")),
    ("hmc7043_gpo", 0, Pins("L14"), IOStandard("LVCMOS33")),

    ("rtm_fpga_sysref", 0,
        Subsignal("p", Pins("R3")),
        Subsignal("n", Pins("T2")),
        IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")
    ),
    ("rtm_fpga_sysref", 1,
        Subsignal("p", Pins("R5")),
        Subsignal("n", Pins("T5")),
        IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")
    ),

    # clock mux
    ("clk_src_ext_sel", 0, Pins("R6"), IOStandard("LVCMOS25")),

    # DACs
    ("ad9154_spi", 0,
        Subsignal("clk", Pins("V4"), Misc("PULLDOWN=TRUE")),
        Subsignal("cs_n", Pins("U4"), Misc("PULLUP=TRUE")),
        Subsignal("mosi", Pins("P6"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("P5"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS25")
    ),
    ("ad9154_txen", 0, Pins("U6 U5"), IOStandard("LVCMOS25")),
    ("ad9154_rst_n", 0, Pins("T3"), IOStandard("LVCMOS25")),
    ("ad9154_spi", 1,
        Subsignal("clk", Pins("K2"), Misc("PULLDOWN=TRUE")),
        Subsignal("cs_n", Pins("J4"), Misc("PULLUP=TRUE")),
        Subsignal("mosi", Pins("K1"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("K3"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS25")
    ),
    ("ad9154_txen", 1, Pins("L2 L4"), IOStandard("LVCMOS25")),
    ("ad9154_rst_n", 1, Pins("J5"), IOStandard("LVCMOS25")),

    ("i2c", 0,
        Subsignal("scl", Pins("P15")),
        Subsignal("sda", Pins("P16")),
        IOStandard("LVCMOS33")
    ),

    ("filtered_clk_sel", 0, Pins("M5"), IOStandard("LVCMOS25")),
    ("si5324", 0,
        Subsignal("rst_n", Pins("C14"), IOStandard("LVCMOS25")),
        Subsignal("int", Pins("V6"), IOStandard("LVCMOS25"))
    ),
    ("si5324_clkin", 0,
        Subsignal("p", Pins("M2")),
        Subsignal("n", Pins("M1")),
        IOStandard("LVDS_25"),
    ),
    ("cdr_clk_clean", 0,
        Subsignal("p", Pins("B6")),
        Subsignal("n", Pins("B5"))
    ),
    ("cdr_clk_clean_fabric", 0,
        Subsignal("p", Pins("R2")),
        Subsignal("n", Pins("R1")),
        IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")
    ),
    ("rtm_amc_link", 0,
        Subsignal("txp", Pins("H2")),
        Subsignal("txn", Pins("H1")),
        Subsignal("rxp", Pins("E4")),
        Subsignal("rxn", Pins("E3"))
    ),
    # SATA connector J13
    ("sata", 0,
        Subsignal("txp", Pins("B2")),
        Subsignal("txn", Pins("B1")),
        Subsignal("rxp", Pins("G4")),
        Subsignal("rxn", Pins("G3"))
    ),

    # Bogus AFE pins for testing (TODO)
    ("allaki0_rfsw0", 0, Pins("E18"), IOStandard("LVCMOS25")),
    ("allaki0_rfsw1", 0, Pins("F17"), IOStandard("LVCMOS25")),
]

class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20.0

    def __init__(self):
        chip = "xc7a50t-csg325-3"
        XilinxPlatform.__init__(self, chip, _io,
                                toolchain="vivado", name="sayma_rtm")
        self.toolchain.bitstream_commands.extend([
            "set_property BITSTREAM.CONFIG.OVERTEMPPOWERDOWN Enable [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS True [current_design]",
            "set_property CFGBVS VCCO [current_design]",
            "set_property CONFIG_VOLTAGE 3.3 [current_design]",
        ])



"""
Clock domain crossing module
"""
from math import gcd

from migen.fhdl.structure import *
from migen.fhdl.module import Module
from migen.fhdl.specials import Special, Memory
from migen.fhdl.bitcontainer import value_bits_sign
from migen.fhdl.decorators import ClockDomainsRenamer
from migen.genlib.misc import WaitTimer
from migen.genlib.resetsync import AsyncResetSynchronizer


class MultiRegImpl(Module):
    def __init__(self, i, o, odomain, n, reset=0):
        self.i = i
        self.o = o
        self.odomain = odomain

        w, signed = value_bits_sign(self.i)
        self.regs = [Signal((w, signed), reset=reset, reset_less=True)
                for i in range(n)]

        ###

        sd = getattr(self.sync, self.odomain)
        src = self.i
        for reg in self.regs:
            sd += reg.eq(src)
            src = reg
        self.comb += self.o.eq(src)
        for reg in self.regs:
            reg.attr.add("no_retiming")


class MultiReg(Special):
    def __init__(self, i, o, odomain="sys", n=2, reset=0):
        Special.__init__(self)
        self.i = wrap(i)
        self.o = wrap(o)
        self.odomain = odomain
        self.n = n
        self.reset = reset

    def iter_expressions(self):
        yield self, "i", SPECIAL_INPUT
        yield self, "o", SPECIAL_OUTPUT

    def rename_clock_domain(self, old, new):
        Special.rename_clock_domain(self, old, new)
        if self.odomain == old:
            self.odomain = new

    def list_clock_domains(self):
        r = Special.list_clock_domains(self)
        r.add(self.odomain)
        return r

    @staticmethod
    def lower(dr):
        return MultiRegImpl(dr.i, dr.o, dr.odomain, dr.n, dr.reset)


class PulseSynchronizer(Module):
    def __init__(self, idomain, odomain):
        self.i = Signal()
        self.o = Signal()

        ###

        toggle_i = Signal(reset_less=True)
        toggle_o = Signal()  # registered reset_less by MultiReg
        toggle_o_r = Signal(reset_less=True)

        sync_i = getattr(self.sync, idomain)
        sync_o = getattr(self.sync, odomain)

        sync_i += If(self.i, toggle_i.eq(~toggle_i))
        self.specials += MultiReg(toggle_i, toggle_o, odomain)
        sync_o += toggle_o_r.eq(toggle_o)
        self.comb += self.o.eq(toggle_o ^ toggle_o_r)


class BusSynchronizer(Module):
    """Clock domain transfer of several bits at once.

    Ensures that all the bits form a single word that was present
    synchronously in the input clock domain (unlike direct use of
    ``MultiReg``)."""
    def __init__(self, width, idomain, odomain, timeout=128):
        self.i = Signal(width)
        self.o = Signal(width, reset_less=True)

        if width == 1:
            self.specials += MultiReg(self.i, self.o, odomain)
        else:
            sync_i = getattr(self.sync, idomain)
            sync_o = getattr(self.sync, odomain)

            starter = Signal(reset=1)
            sync_i += starter.eq(0)
            self.submodules._ping = PulseSynchronizer(idomain, odomain)
            # Extra flop on i->o to avoid race between data and request
            # https://github.com/m-labs/nmigen/pull/40#issuecomment-484166790
            ping_o = Signal()
            sync_o += ping_o.eq(self._ping.o)
            self.submodules._pong = PulseSynchronizer(odomain, idomain)
            self.submodules._timeout = ClockDomainsRenamer(idomain)(
                WaitTimer(timeout))
            self.comb += [
                self._timeout.wait.eq(~self._ping.i),
                self._ping.i.eq(starter | self._pong.o | self._timeout.done),
                self._pong.i.eq(ping_o)
            ]

            ibuffer = Signal(width, reset_less=True)
            obuffer = Signal(width)  # registered reset_less by MultiReg
            sync_i += If(self._pong.o, ibuffer.eq(self.i))
            ibuffer.attr.add("no_retiming")
            self.specials += MultiReg(ibuffer, obuffer, odomain)
            sync_o += If(ping_o, self.o.eq(obuffer))


class BlindTransfer(Module):
    """
    PulseSynchronizer but with the input "blinded" until the pulse
    is received in the destination domain.

    This avoids situations where two pulses in the input domain
    at a short interval (shorter than the destination domain clock
    period) causes two toggles of the internal PulseSynchronizer
    signal and no output pulse being emitted.

    With this module, any activity in the input domain will generate
    at least one pulse at the output.

    An optional data word can be transferred with each registered pulse.
    """
    def __init__(self, idomain, odomain, data_width=0):
        self.i = Signal()
        self.o = Signal()
        if data_width:
            self.data_i = Signal(data_width)
            self.data_o = Signal(data_width, reset_less=True)

        # # #

        ps = PulseSynchronizer(idomain, odomain)
        ps_ack = PulseSynchronizer(odomain, idomain)
        self.submodules += ps, ps_ack
        blind = Signal()
        isync = getattr(self.sync, idomain)
        isync += [
            If(self.i, blind.eq(1)),
            If(ps_ack.o, blind.eq(0))
        ]
        self.comb += [
            ps.i.eq(self.i & ~blind),
            ps_ack.i.eq(ps.o),
            self.o.eq(ps.o)
        ]

        if data_width:
            bxfer_data = Signal(data_width, reset_less=True)
            isync += If(ps.i, bxfer_data.eq(self.data_i))
            bxfer_data.attr.add("no_retiming")
            self.specials += MultiReg(bxfer_data, self.data_o,
                                      odomain=odomain)


class GrayCounter(Module):
    def __init__(self, width):
        self.ce = Signal()
        self.q = Signal(width)
        self.q_next = Signal(width)
        self.q_binary = Signal(width)
        self.q_next_binary = Signal(width)

        ###

        self.comb += [
            If(self.ce,
                self.q_next_binary.eq(self.q_binary + 1)
            ).Else(
                self.q_next_binary.eq(self.q_binary)
            ),
            self.q_next.eq(self.q_next_binary ^ self.q_next_binary[1:])
        ]
        self.sync += [
            self.q_binary.eq(self.q_next_binary),
            self.q.eq(self.q_next)
        ]


class GrayDecoder(Module):
    def __init__(self, width):
        self.i = Signal(width)
        self.o = Signal(width, reset_less=True)

        # # #

        o_comb = Signal(width)
        self.comb += o_comb[-1].eq(self.i[-1])
        for i in reversed(range(width-1)):
            self.comb += o_comb[i].eq(o_comb[i+1] ^ self.i[i])
        self.sync += self.o.eq(o_comb)


class ElasticBuffer(Module):
    def __init__(self, width, depth, idomain, odomain):
        self.din = Signal(width)
        self.dout = Signal(width)

        # # #

        reset = Signal()
        cd_write = ClockDomain()
        cd_read = ClockDomain()
        self.comb += [
            cd_write.clk.eq(ClockSignal(idomain)),
            cd_read.clk.eq(ClockSignal(odomain)),
            reset.eq(ResetSignal(idomain) | ResetSignal(odomain))
        ]
        self.specials += [
            AsyncResetSynchronizer(cd_write, reset),
            AsyncResetSynchronizer(cd_read, reset)
        ]
        self.clock_domains += cd_write, cd_read

        wrpointer = Signal(max=depth, reset=depth//2)
        rdpointer = Signal(max=depth)

        storage = Memory(width, depth)
        self.specials += storage

        wrport = storage.get_port(write_capable=True, clock_domain="write")
        rdport = storage.get_port(clock_domain="read")
        self.specials += wrport, rdport

        self.sync.write += wrpointer.eq(wrpointer + 1)
        self.sync.read += rdpointer.eq(rdpointer + 1)

        self.comb += [
            wrport.we.eq(1),
            wrport.adr.eq(wrpointer),
            wrport.dat_w.eq(self.din),

            rdport.adr.eq(rdpointer),
            self.dout.eq(rdport.dat_r)
        ]


def lcm(a, b):
    """Compute the lowest common multiple of a and b"""
    return (a*b)//gcd(a, b)


class Gearbox(Module):
    def __init__(self, iwidth, idomain, owidth, odomain):
        self.i = Signal(iwidth)
        self.o = Signal(owidth, reset_less=True)

        # # #

        rst = Signal()
        cd_write = ClockDomain()
        cd_read = ClockDomain()
        self.comb += [
            rst.eq(ResetSignal(idomain) | ResetSignal(odomain)),
            cd_write.clk.eq(ClockSignal(idomain)),
            cd_read.clk.eq(ClockSignal(odomain)),
            cd_write.rst.eq(rst),
            cd_read.rst.eq(rst)
        ]
        self.clock_domains += cd_write, cd_read

        storage = Signal(2*lcm(iwidth, owidth), reset_less=True)
        wrchunks = len(storage)//iwidth
        rdchunks = len(storage)//owidth
        wrpointer = Signal(max=wrchunks, reset=0 if iwidth > owidth else wrchunks//2)
        rdpointer = Signal(max=rdchunks, reset=rdchunks//2 if iwidth > owidth else 0)

        self.sync.write += \
            If(wrpointer == wrchunks-1,
                wrpointer.eq(0)
            ).Else(
                wrpointer.eq(wrpointer + 1)
            )
        cases = {}
        for i in range(wrchunks):
            cases[i] = [storage[iwidth*i:iwidth*(i+1)].eq(self.i)]
        self.sync.write += Case(wrpointer, cases)


        self.sync.read += \
            If(rdpointer == rdchunks-1,
                rdpointer.eq(0)
            ).Else(
                rdpointer.eq(rdpointer + 1)
            )
        cases = {}
        for i in range(rdchunks):
            cases[i] = [self.o.eq(storage[owidth*i:owidth*(i+1)])]
        self.sync.read += Case(rdpointer, cases)



from operator import itemgetter

from migen.fhdl.structure import *
from migen.fhdl.structure import _Value
from migen.fhdl.module import *
from migen.fhdl.bitcontainer import bits_for, value_bits_sign
from migen.fhdl.tools import *
from migen.fhdl.tracer import get_obj_var_name
from migen.fhdl.verilog import _printexpr as verilog_printexpr


__all__ = ["TSTriple", "Instance", "Memory",
    "READ_FIRST", "WRITE_FIRST", "NO_CHANGE"]


class Special(DUID):
    def iter_expressions(self):
        for x in []:
            yield x

    def rename_clock_domain(self, old, new):
        for obj, attr, direction in self.iter_expressions():
            rename_clock_domain_expr(getattr(obj, attr), old, new)

    def list_clock_domains(self):
        r = set()
        for obj, attr, direction in self.iter_expressions():
            r |= list_clock_domains_expr(getattr(obj, attr))
        return r

    def list_ios(self, ins, outs, inouts):
        r = set()
        for obj, attr, direction in self.iter_expressions():
            if (direction == SPECIAL_INPUT and ins) \
              or (direction == SPECIAL_OUTPUT and outs) \
              or (direction == SPECIAL_INOUT and inouts):
                signals = list_signals(getattr(obj, attr))
                r.update(signals)
        return r


class _TristateModule(Module):
    def __init__(self, target, o, oe, i):
        self.comb += [
            target.o.eq(o),
            target.oe.eq(oe)
        ]
        if i is not None:
            self.comb += i.eq(target.i)


class Tristate(Special):
    def __init__(self, target, o, oe, i=None):
        Special.__init__(self)
        self._isrecord = hasattr(target, "o") and hasattr(target, "oe")
        if not self._isrecord:
            self.target = wrap(target)
        else:
            self.target = target
            self._target_o = target.o
            self._target_oe = target.oe
            self._target_i = getattr(target, "i", None)
            if i is not None and not hasattr(target, "i"):
                raise ValueError("target has to have i field if parameter i is not None")
        self.o = wrap(o)
        self.oe = wrap(oe)
        self.i = wrap(i) if i is not None else None

    def iter_expressions(self):
        if not self._isrecord:
            tri_attr_context = [
                ("target", SPECIAL_INOUT)
            ]
        else:
            tri_attr_context = [
                ("_target_o", SPECIAL_OUTPUT),
                ("_target_oe", SPECIAL_OUTPUT),
                ("_target_i", SPECIAL_INPUT)
            ]
        tri_attr_context += [
            ("o", SPECIAL_INPUT),
            ("oe", SPECIAL_INPUT),
            ("i", SPECIAL_OUTPUT)
        ]
        for attr, target_context in tri_attr_context:
            if getattr(self, attr) is not None:
                yield self, attr, target_context

    @staticmethod
    def lower(tristate):
        if not tristate._isrecord:
            return None
        else:
            return _TristateModule(tristate.target, tristate.o, tristate.oe, tristate.i)

    @staticmethod
    def emit_verilog(tristate, ns, add_data_file):
        assert(not tristate._isrecord)
        def pe(e):
            return verilog_printexpr(ns, e)[0]
        w, s = value_bits_sign(tristate.target)
        r = "assign " + pe(tristate.target) + " = " \
            + pe(tristate.oe) + " ? " + pe(tristate.o) \
            + " : " + str(w) + "'bz;\n"
        if tristate.i is not None:
            r += "assign " + pe(tristate.i) + " = " + pe(tristate.target) + ";\n"
        r += "\n"
        return r


class TSTriple:
    def __init__(self, bits_sign=None, min=None, max=None, reset_o=0, reset_oe=0, reset_i=0,
                 name=None):
        self.o = Signal(bits_sign, min=min, max=max, reset=reset_o,
                        name=None if name is None else name + "_o")
        self.oe = Signal(reset=reset_oe,
                         name=None if name is None else name + "_oe")
        self.i = Signal(bits_sign, min=min, max=max, reset=reset_i,
                        name=None if name is None else name + "_i")

    def get_tristate(self, target):
        return Tristate(target, self.o, self.oe, self.i)

    def __len__(self):
        return len(self.o)


class Instance(Special):
    class _IO:
        def __init__(self, name, expr=None):
            self.name = name
            if expr is None:
                expr = Signal()
            self.expr = wrap(expr)
    class Input(_IO):
        pass
    class Output(_IO):
        pass
    class InOut(_IO):
        pass
    class Parameter:
        def __init__(self, name, value):
            self.name = name
            if isinstance(value, (int, bool)):
                value = Constant(value)
            self.value = value
    class PreformattedParam(str):
        pass

    def __init__(self, of, *items, name="", synthesis_directive=None,
            attr=None, **kwargs):
        Special.__init__(self)
        self.of = of
        if name:
            self.name_override = name
        else:
            self.name_override = of
        self.items = list(items)
        self.synthesis_directive = synthesis_directive
        if attr is None:
            attr = set()
        self.attr = attr
        for k, v in sorted(kwargs.items(), key=itemgetter(0)):
            try:
                item_type, item_name = k.split("_", maxsplit=1)
            except ValueError:
                raise TypeError("Wrong format for value '" + str(k) +
                                "', format should be 'type_name'")

            item_class = {
                "i": Instance.Input,
                "o": Instance.Output,
                "io": Instance.InOut,
                "p": Instance.Parameter
            }[item_type]
            self.items.append(item_class(item_name, v))

    def get_io(self, name):
        for item in self.items:
            if isinstance(item, Instance._IO) and item.name == name:
                return item.expr

    def iter_expressions(self):
        for item in self.items:
            if isinstance(item, Instance.Input):
                yield item, "expr", SPECIAL_INPUT
            elif isinstance(item, Instance.Output):
                yield item, "expr", SPECIAL_OUTPUT
            elif isinstance(item, Instance.InOut):
                yield item, "expr", SPECIAL_INOUT

    @staticmethod
    def emit_verilog(instance, ns, add_data_file):
        r = instance.of + " "
        parameters = list(filter(lambda i: isinstance(i, Instance.Parameter), instance.items))
        if parameters:
            r += "#(\n"
            firstp = True
            for p in parameters:
                if not firstp:
                    r += ",\n"
                firstp = False
                r += "\t." + p.name + "("
                if isinstance(p.value, Constant):
                    r += verilog_printexpr(ns, p.value)[0]
                elif isinstance(p.value, float):
                    r += str(p.value)
                elif isinstance(p.value, Instance.PreformattedParam):
                    r += p.value
                elif isinstance(p.value, str):
                    r += "\"" + p.value + "\""
                else:
                    raise TypeError
                r += ")"
            r += "\n) "
        r += ns.get_name(instance)
        if parameters: r += " "
        r += "(\n"
        firstp = True
        for p in instance.items:
            if isinstance(p, Instance._IO):
                name_inst = p.name
                name_design = verilog_printexpr(ns, p.expr)[0]
                if not firstp:
                    r += ",\n"
                firstp = False
                r += "\t." + name_inst + "(" + name_design + ")"
        if not firstp:
            r += "\n"
        if instance.synthesis_directive is not None:
            synthesis_directive = "/* synthesis {} */".format(instance.synthesis_directive)
            r += ")" + synthesis_directive + ";\n\n"
        else:
            r += ");\n\n"
        return r


(READ_FIRST, WRITE_FIRST, NO_CHANGE) = range(3)


class _MemoryPort(Special):
    def __init__(self, adr, dat_r, we=None, dat_w=None,
      async_read=False, re=None, we_granularity=0, mode=WRITE_FIRST,
      clock_domain="sys"):
        Special.__init__(self)
        self.adr = adr
        self.dat_r = dat_r
        self.we = we
        self.dat_w = dat_w
        self.async_read = async_read
        self.re = re
        self.we_granularity = we_granularity
        self.mode = mode
        self.clock = ClockSignal(clock_domain)

    def iter_expressions(self):
        for attr, target_context in [
          ("adr", SPECIAL_INPUT),
          ("we", SPECIAL_INPUT),
          ("dat_w", SPECIAL_INPUT),
          ("re", SPECIAL_INPUT),
          ("dat_r", SPECIAL_OUTPUT),
          ("clock", SPECIAL_INPUT)]:
            yield self, attr, target_context

    @staticmethod
    def emit_verilog(port, ns, add_data_file):
        return ""  # done by parent Memory object


class _MemoryLocation(_Value):
    def __init__(self, memory, index):
        _Value.__init__(self)
        self.memory = memory
        self.index = wrap(index)


class Memory(Special):
    def __init__(self, width, depth, init=None, name=None):
        Special.__init__(self)
        self.width = width
        self.depth = depth
        self.ports = []
        self.init = init
        self.name_override = get_obj_var_name(name, "mem")

    def __getitem__(self, index):
        # simulation only
        return _MemoryLocation(self, index)

    def get_port(self, write_capable=False, async_read=False,
      has_re=False, we_granularity=0, mode=WRITE_FIRST,
      clock_domain="sys"):
        if we_granularity >= self.width:
            we_granularity = 0
        adr = Signal(max=self.depth)
        dat_r = Signal(self.width)
        if write_capable:
            if we_granularity:
                we = Signal(self.width//we_granularity)
            else:
                we = Signal()
            dat_w = Signal(self.width)
        else:
            we = None
            dat_w = None
        if has_re:
            re = Signal()
        else:
            re = None
        mp = _MemoryPort(adr, dat_r, we, dat_w,
          async_read, re, we_granularity, mode,
          clock_domain)
        self.ports.append(mp)
        return mp

    @staticmethod
    def emit_verilog(memory, ns, add_data_file):
        r = ""
        def gn(e):
            if isinstance(e, Memory):
                return ns.get_name(e)
            else:
                return verilog_printexpr(ns, e)[0]
        adrbits = bits_for(memory.depth-1)

        r += "reg [" + str(memory.width-1) + ":0] " \
            + gn(memory) \
            + "[0:" + str(memory.depth-1) + "];\n"

        adr_regs = {}
        data_regs = {}
        for port in memory.ports:
            if not port.async_read:
                if port.mode == WRITE_FIRST:
                    adr_reg = Signal(name_override="memadr")
                    r += "reg [" + str(adrbits-1) + ":0] " \
                        + gn(adr_reg) + ";\n"
                    adr_regs[id(port)] = adr_reg
                else:
                    data_reg = Signal(name_override="memdat")
                    r += "reg [" + str(memory.width-1) + ":0] " \
                        + gn(data_reg) + ";\n"
                    data_regs[id(port)] = data_reg

        for port in memory.ports:
            r += "always @(posedge " + gn(port.clock) + ") begin\n"
            if port.we is not None:
                if port.we_granularity:
                    n = memory.width//port.we_granularity
                    for i in range(n):
                        m = i*port.we_granularity
                        M = (i+1)*port.we_granularity-1
                        sl = "[" + str(M) + ":" + str(m) + "]"
                        r += "\tif (" + gn(port.we) + "[" + str(i) + "])\n"
                        r += "\t\t" + gn(memory) + "[" + gn(port.adr) + "]" + sl + " <= " + gn(port.dat_w) + sl + ";\n"
                else:
                    r += "\tif (" + gn(port.we) + ")\n"
                    r += "\t\t" + gn(memory) + "[" + gn(port.adr) + "] <= " + gn(port.dat_w) + ";\n"
            if not port.async_read:
                if port.mode == WRITE_FIRST:
                    rd = "\t" + gn(adr_regs[id(port)]) + " <= " + gn(port.adr) + ";\n"
                else:
                    bassign = gn(data_regs[id(port)]) + " <= " + gn(memory) + "[" + gn(port.adr) + "];\n"
                    if port.mode == READ_FIRST:
                        rd = "\t" + bassign
                    elif port.mode == NO_CHANGE:
                        rd = "\tif (!" + gn(port.we) + ")\n" \
                          + "\t\t" + bassign
                if port.re is None:
                    r += rd
                else:
                    r += "\tif (" + gn(port.re) + ")\n"
                    r += "\t" + rd.replace("\n\t", "\n\t\t")
            r += "end\n\n"

        for port in memory.ports:
            if port.async_read:
                r += "assign " + gn(port.dat_r) + " = " + gn(memory) + "[" + gn(port.adr) + "];\n"
            else:
                if port.mode == WRITE_FIRST:
                    r += "assign " + gn(port.dat_r) + " = " + gn(memory) + "[" + gn(adr_regs[id(port)]) + "];\n"
                else:
                    r += "assign " + gn(port.dat_r) + " = " + gn(data_regs[id(port)]) + ";\n"
        r += "\n"

        if memory.init is not None:
            content = ""
            for d in memory.init:
                content += "{:x}\n".format(d)
            memory_filename = add_data_file(gn(memory) + ".init", content)

            r += "initial begin\n"
            r += "\t$readmemh(\"" + memory_filename + "\", " + gn(memory) + ");\n"
            r += "end\n\n"

        return r



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
        ("clk64", 0,
            Subsignal("p", Pins("R7")),
            Subsignal("n", Pins("T7")),
            IOStandard("LVDS_33"),
            Misc("DIFF_TERM=TRUE"),
        ),

        ("pps", 0, Pins("M14"), Misc("TIG")),
        ("reset_n", 0, Pins("D5"), Misc("TIG")),
        ("codec_reset", 0, Pins("B14")),
        # recycles fpga_cfg_cclk for reset from fw
        ("ext_reset", 0, Pins("R14")),

        ("i2c", 0,
            Subsignal("sda", Pins("T13")),
            Subsignal("scl", Pins("R13")),
        ),

        ("cgen", 0,
            Subsignal("st_ld", Pins("M13")),
            Subsignal("st_refmon", Pins("J14")),
            Subsignal("st_status", Pins("P6")),
            Subsignal("ref_sel", Pins("T2")),
            Subsignal("sync_b", Pins("H15")),
        ),

        ("fx2_ifclk", 0, Pins("T8")),
        ("fx2_gpif", 0,
            Subsignal("d", Pins("P8 P9 N9 T9 R9 P11 P13 N12 "
                                "T3 R3 P5 N6 T6 T5 N8 P7")),
            Subsignal("ctl", Pins("M7 M9 M11 P12")),
            Subsignal("slwr", Pins("T4")),  # rdy0
            Subsignal("slrd", Pins("R5")),  # rdy1
            # Subsignal("rdy2", Pins("T10")),
            # Subsignal("rdy3", Pins("N11")),
            # Subsignal("cs", Pins("P12")),
            Subsignal("sloe", Pins("R11")),
            Subsignal("pktend", Pins("P10")),
            Subsignal("adr", Pins("T11 H16")),
        ),

        ("user_led", 0, Pins("P4"), Misc("TIG")),
        ("user_led", 1, Pins("N4"), Misc("TIG")),
        ("user_led", 2, Pins("R2"), Misc("TIG")),

        ("debug_clk", 0, Pins("K15 K14")),
        ("debug", 0, Pins(
            "K16 J16 C16 C15 E13 D14 D16 D15 "
            "E14 F13 G13 F14 E16 F15 H13 G14 "
            "G16 F16 J12 J13 L14 L16 M15 M16 "
            "L13 K13 P16 N16 R15 P15 N13 N14")),

        ("adc", 0,
            Subsignal("sync", Pins("D10")),
            Subsignal("d", Pins("A4 B3 A3 D9 C10 A9 C9 D8 "
                                "C8 B8 A8 B15")),
        ),
        ("dac", 0,
            Subsignal("blank", Pins("K1")),
            Subsignal("sync", Pins("J2")),
            Subsignal("d", Pins("J1 H3 J3 G2 H1 N3 M4 R1 "
                                "P2 P1 M1 N1 M3 L4")),
        ),
        ("codec_spi", 0,
            Subsignal("sclk", Pins("K3")),
            Subsignal("sen", Pins("D13")),
            Subsignal("mosi", Pins("C13")),
            Subsignal("miso", Pins("G4")),
        ),

        ("aux_spi", 0,
            Subsignal("sen", Pins("C12")),
            Subsignal("sclk", Pins("D12")),
            Subsignal("miso", Pins("J5")),
        ),
        ("rx_io", 0, Pins("D7 C6 A6 B6 E9 A7 C7 B10 "
                          "A10 C11 A11 D11 B12 A12 A14 A13")),
        ("tx_io", 0, Pins("K4 L3 L2 F1 F3 G3 E3 E2 "
                          "E4 F4 D1 E1 D4 D3 C2 C1")),
        ("rx_spi", 0,
            Subsignal("miso", Pins("E6")),
            Subsignal("sen", Pins("B4")),
            Subsignal("mosi", Pins("A5")),
            Subsignal("sclk", Pins("C5")),
        ),
        ("tx_spi", 0,
            Subsignal("miso", Pins("J4")),
            Subsignal("sen", Pins("N2")),
            Subsignal("mosi", Pins("L1")),
            Subsignal("sclk", Pins("G1")),
        ),

        # these are just for information. do not request.
        ("mystery_bus", 0, Pins("C4 E7")),
        ("fpga_cfg",
            Subsignal("din", Pins("T14")),
            Subsignal("cclk", Pins("R14")),
            Subsignal("init_b", Pins("T12")),
            Subsignal("prog_b", Pins("A2")),
            Subsignal("done", Pins("T15")),
        ),
        ("jtag",
            Subsignal("tms", Pins("B2")),
            Subsignal("tdo", Pins("B16")),
            Subsignal("tdi", Pins("B1")),
            Subsignal("tck", Pins("A15")),
        ),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk64"
    default_clk_period = 15.625

    def __init__(self):
        XilinxPlatform.__init__(self, "xc3s1400a-ft256-4", _io)
        self.toolchain.bitgen_opt = "-g LCK_cycle:6 -g Binary:Yes -w -g UnusedPin:PullUp"

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)

        self.add_platform_command("""
TIMESPEC TS_Pad2Pad = FROM PADS TO PADS 7 ns;
""")

        try:
            ifclk = self.lookup_request("fx2_ifclk")
            gpif = self.lookup_request("fx2_gpif")
            for i, d in [(gpif.d, "in"), (gpif.d, "out"),
                    (gpif.ctl, "in"), (gpif.adr, "out"),
                    (gpif.slwr, "out"), (gpif.sloe, "out"),
                    (gpif.slrd, "out"), (gpif.pktend, "out")]:
                if len(i) > 1:
                    q = "(*)"
                else:
                    q = ""
                self.add_platform_command("""
INST "{i}%s" TNM = gpif_net_%s;
""" % (q, d), i=i)
            self.add_platform_command("""
NET "{ifclk}" TNM_NET = "GRPifclk";
TIMESPEC "TSifclk" = PERIOD "GRPifclk" 20833 ps HIGH 50%;
TIMEGRP "gpif_net_in" OFFSET = IN 5 ns VALID 10 ns BEFORE "{ifclk}" RISING;
TIMEGRP "gpif_net_out" OFFSET = OUT 7 ns AFTER "{ifclk}" RISING;
""", ifclk=ifclk)
        except ConstraintError:
             pass



# This file is Copyright (c) 2018 William D. Jones <thor0505@comcast.net>
# License: BSD

from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform, XC3SProg, VivadoProgrammer

_io = [
    ("user_led", 0, Pins("E18"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("F13"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("E13"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("H15"), IOStandard("LVCMOS33")),

    ("rgb_led", 0,
        Subsignal("r", Pins("J15")),
        Subsignal("g", Pins("G17")),
        Subsignal("b", Pins("F15")),
        IOStandard("LVCMOS33")
    ),

    ("rgb_led", 1,
        Subsignal("r", Pins("E15")),
        Subsignal("g", Pins("F18")),
        Subsignal("b", Pins("E14")),
        IOStandard("LVCMOS33")
    ),

    ("user_sw", 0, Pins("H14"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("H18"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("G18"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("M5"), IOStandard("LVCMOS33")),

    ("user_btn", 0, Pins("G15"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("K16"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("J16"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("H13"), IOStandard("LVCMOS33")),

    ("clk100", 0, Pins("R2"), IOStandard("SSTL135")),

    ("cpu_reset", 0, Pins("C18"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("R12")),
        Subsignal("rx", Pins("V12")),
        IOStandard("LVCMOS33")),

    ("spi", 0,
        Subsignal("clk", Pins("G16")),
        Subsignal("cs_n", Pins("H16")),
        Subsignal("mosi", Pins("H17")),
        Subsignal("miso", Pins("K14")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("M13")),
        Subsignal("dq", Pins("K17", "K18", "L14", "M15")),
        IOStandard("LVCMOS33")
    ),
    ("spiflash", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("M13")),
        Subsignal("mosi", Pins("K17")),
        Subsignal("miso", Pins("K18")),
        Subsignal("wp", Pins("L14")),
        Subsignal("hold", Pins("M15")),
        IOStandard("LVCMOS33")
    ),

    ("ddram", 0,
        Subsignal("a", Pins(
            "U2 R4 V2 V4 T3 R7 V6 T6",
            "U7 V7 P6 T5 R6 U6"),
            IOStandard("SSTL135")),
        Subsignal("ba", Pins("V5 T1 U3"), IOStandard("SSTL135")),
        Subsignal("ras_n", Pins("U1"), IOStandard("SSTL135")),
        Subsignal("cas_n", Pins("V3"), IOStandard("SSTL135")),
        Subsignal("we_n", Pins("P7"), IOStandard("SSTL135")),
        Subsignal("cs_n", Pins("R3"), IOStandard("SSTL135")),
        Subsignal("dm", Pins("K4 M3"), IOStandard("SSTL135")),
        Subsignal("dq", Pins(
            "K2 K3 L4 M6 K6 M4 L5 L6",
            "N4 R1 N1 N5 M2 P1 M1 P2"),
            IOStandard("SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("K1 N3"), IOStandard("DIFF_SSTL135")),
        Subsignal("dqs_n", Pins("L1 N2"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_p", Pins("R5"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_n", Pins("T4"), IOStandard("DIFF_SSTL135")),
        Subsignal("cke", Pins("T2"), IOStandard("SSTL135")),
        Subsignal("odt", Pins("P5"), IOStandard("SSTL135")),
        Subsignal("reset_n", Pins("J6"), IOStandard("SSTL135")),
        Misc("SLEW=FAST"),
    ),

]

_connectors = [
    ("pmoda", "L17 L18 M14 N14 M16 M17 M18 N18"),
    ("pmodb", "P17 P18 R18 T18 P14 P15 N15 P16"),
    ("pmodc", "U15 V16 U17 U18 U16 P13 R13 V14"),
    ("pmodd", "V15 U12 V13 T12 T13 R11 T11 U11")
]

class Platform(XilinxPlatform):
    default_clk_name = "clk100"
    default_clk_period = 10.0

    def __init__(self, toolchain="vivado", programmer="vivado"):
        XilinxPlatform.__init__(self, "xc7s50csga324-1", _io, _connectors,
                                toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.programmer = programmer
        self.add_platform_command("set_property INTERNAL_VREF 0.675 [get_iobanks 34]")

    def create_programmer(self):
        if self.programmer == "xc3sprog":
            return XC3SProg("nexys4")
        elif self.programmer == "vivado":
            return VivadoProgrammer(flash_part="n25q128-3.3v-spi-x1_x2_x4")
        else:
            raise ValueError("{} programmer is not supported"
                             .format(self.programmer))



from math import gcd
import collections.abc


def flat_iteration(l):
    for element in l:
        if isinstance(element, collections.abc.Iterable):
            for element2 in flat_iteration(element):
                yield element2
        else:
            yield element


def xdir(obj, return_values=False):
    for attr in dir(obj):
        if attr[:2] != "__" and attr[-2:] != "__":
            if return_values:
                yield attr, getattr(obj, attr)
            else:
                yield attr


def gcd_multiple(numbers):
    l = len(numbers)
    if l == 1:
        return numbers[0]
    else:
        s = l//2
        return gcd(gcd_multiple(numbers[:s]), gcd_multiple(numbers[s:]))



import os
import subprocess

from migen.build.generic_programmer import GenericProgrammer
from migen.build import tools


class LatticeProgrammer(GenericProgrammer):
    needs_bitreverse = False

    def __init__(self, xcf_template):
        self.xcf_template = xcf_template

    def load_bitstream(self, bitstream_file):
        xcf_file = bitstream_file.replace(".bit", ".xcf")
        xcf_content = self.xcf_template.format(bitstream_file=bitstream_file)
        tools.write_to_file(xcf_file, xcf_content)
        subprocess.call(["pgrcmd", "-infile", xcf_file])


class IceStormProgrammer(GenericProgrammer):
    needs_bitreverse = False

    def flash(self, address, bitstream_file):
        subprocess.call(["iceprog", "-o", str(address), bitstream_file])

    def load_bitstream(self, bitstream_file):
        subprocess.call(["iceprog", "-S", bitstream_file])


class IceBurnProgrammer(GenericProgrammer):
    def __init__(self, iceburn_path):
        GenericProgrammer.__init__(self)
        self.iceburn = iceburn_path

    needs_bitreverse = False

    def load_bitstream(self, bitstream_file):
        subprocess.call([self.iceburn, "-evw", bitstream_file])


class TinyFpgaBProgrammer(GenericProgrammer):
    needs_bitreverse = False

    # The default flash address you probably want is 0x30000; the image at
    # address 0 is for the bootloader.
    def flash(self, address, bitstream_file):
        subprocess.call(["tinyfpgab", "-a", str(address), "-p",
                        bitstream_file])

    # Force user image to boot if a user reset tinyfpga, the bootloader
    # is active, and the user image need not be reprogrammed.
    def boot(self):
        subprocess.call(["tinyfpgab", "-b"])


# Different bootloader protocol requires different application. In the basic
# case, command-line arguments are the same. Note that this programmer can
# also be used with TinyFPGA B2 if you have updated its bootloader.
class TinyProgProgrammer(GenericProgrammer):
    needs_bitreverse = False

    # You probably want to pass address="None" for this programmer
    # unless you know what you're doing.
    def flash(self, address, bitstream_file, user_data=False):
        if address is None:
            if not user_data:
                # tinyprog looks at spi flash metadata to figure out where to
                # program your bitstream.
                subprocess.call(["tinyprog", "-p", bitstream_file])
            else:
                # Ditto with user data.
                subprocess.call(["tinyprog", "-u", bitstream_file])
        else:
            # Provide override so user can program wherever they wish.
            subprocess.call(["tinyprog", "-a", str(address), "-p",
                            bitstream_file])

    # Force user image to boot if a user reset tinyfpga, the bootloader
    # is active, and the user image need not be reprogrammed.
    def boot(self):
        subprocess.call(["tinyprog", "-b"])


class MyStormProgrammer(GenericProgrammer):
    def __init__(self, serial_port):
        self.serial_port = serial_port

    def load_bitstream(self, bitstream_file):
        import serial
        with serial.Serial(self.serial_port) as port:
            with open(bitstream_file, "rb") as f:
                port.write(f.read())



from migen.build.generic_platform import *
from migen.build.altera import AlteraPlatform
from migen.build.altera.programmer import USBBlaster


_io = [
    ("clk10", 0, Pins("N5"), IOStandard("3.3-V LVTTL")),
    ("clk50", 0, Pins("P11"), IOStandard("3.3-V LVTTL")),
    ("clk50", 1, Pins("N14"), IOStandard("3.3-V LVTTL")),

    ("user_led", 0, Pins("A8"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("A9"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("A10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("B10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("D13"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("C13"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("E14"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("D14"), IOStandard("3.3-V LVTTL")),
    ("user_led", 8, Pins("A11"), IOStandard("3.3-V LVTTL")),
    ("user_led", 9, Pins("B11"), IOStandard("3.3-V LVTTL")),

    ("user_btn", 0, Pins("B8"), IOStandard("3.3-V LVTTL")),
    ("user_btn", 1, Pins("A7"), IOStandard("3.3-V LVTTL")),

    ("user_sw", 0, Pins("C10"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 1, Pins("C11"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 2, Pins("D12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 3, Pins("C12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 4, Pins("A12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 5, Pins("B12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 6, Pins("A13"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 7, Pins("A14"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 8, Pins("B14"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 9, Pins("F15"), IOStandard("3.3-V LVTTL")),

    # 7-segment displays
    ("seven_seg", 0, Pins("C14 E15 C15 C16 E16 D17 C17 D15"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 1, Pins("C18 D18 E18 B16 A17 A18 B17 A16"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 2, Pins("B20 A20 B19 A21 B21 C22 B22 A19"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 3, Pins("F21 E22 E21 C19 C20 D19 E17 D22"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 4, Pins("F18 E20 E19 J18 H19 F19 F20 F17"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 5, Pins("J20 K20 L18 N18 M20 N19 N20 L19"), IOStandard("3.3-V LVTTL")),


    ("gpio_0", 0,
        Pins("V10 W10 V9 W9 V8 W8 V7 W7 W6 V5 W5 AA15 AA14 W13 W12 AB13 AB12 Y11 AB11 W11 AB10 AA10 AA9 Y8 AA8 Y7 AA7 Y6 AA6 Y5 AA5 Y4 AB3 Y3 AB2 AA2"),
        IOStandard("3.3-V LVTTL")
    ),
    ("gpio_1", 0,
        Pins("AB5 AB6 AB7 AB8 AB9 Y10 AA11 AA12 AB17 AA17 AB19 AA19 Y19 AB20 AB21 AA20 F16"),
        IOStandard("3.3-V LVTTL")
    ),

    ("vga_out", 0,
        Subsignal("hsync_n", Pins("N3")),
        Subsignal("vsync_n", Pins("N1")),
        Subsignal("r", Pins("AA1 V1 Y2 Y1")),
        Subsignal("g", Pins("W1 T2 R2 R1")),
        Subsignal("b", Pins("P1 T1 P4 N2")),
        IOStandard("3.3-V LVTTL")
    ),

    ("sdram_clock", 0, Pins("L14"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a", Pins("U17 W19 V18 U18 U19 T18 T19 R18 P18 P19 T20 P20 R20")),
        Subsignal("ba", Pins("T21 T22")),
        Subsignal("cs_n", Pins("U20")),
        Subsignal("cke", Pins("N22")),
        Subsignal("ras_n", Pins("U22")),
        Subsignal("cas_n", Pins("U21")),
        Subsignal("we_n", Pins("V20")),
        Subsignal("dq", Pins("Y21 Y20 AA22 AA21 Y22 W22 W20 V21 P21 J22 H21 H22 G22 G20 G19 F22")),
        Subsignal("dm", Pins("V22 J21")),
        IOStandard("3.3-V LVTTL")
    ),

    ("accelerometer", 0,
        Subsignal("int1", Pins("Y14")),
        Subsignal("int1", Pins("Y13")),
        Subsignal("mosi", Pins("V11")),
        Subsignal("miso", Pins("V12")),
        Subsignal("clk", Pins("AB15")),
        Subsignal("cs_n", Pins("AB16")),
        IOStandard("3.3-V LVTTL")
    )
]


class Platform(AlteraPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20
    create_rbf = False

    def __init__(self):
        AlteraPlatform.__init__(self, "10M50DAF484C7G", _io)

    def create_programmer(self):
        return USBBlaster()



from migen.build.generic_platform import GenericPlatform
from migen.build.lattice import common, diamond, icestorm, trellis


class LatticePlatform(GenericPlatform):
    bitstream_ext = ".bit"

    def __init__(self, *args, toolchain="diamond", **kwargs):
        GenericPlatform.__init__(self, *args, **kwargs)
        if toolchain == "diamond":
            self.toolchain = diamond.LatticeDiamondToolchain()
        elif toolchain == "trellis":
            self.toolchain = trellis.LatticeTrellisToolchain()
        elif toolchain == "icestorm":
            self.bitstream_ext = ".bin"
            self.toolchain = icestorm.LatticeIceStormToolchain()
        else:
            raise ValueError("Unknown toolchain")

    def get_verilog(self, *args, special_overrides=dict(), **kwargs):
        so = dict()  # No common overrides between ECP and ice40.
        so.update(self.toolchain.special_overrides)
        so.update(special_overrides)
        return GenericPlatform.get_verilog(self, *args, special_overrides=so,
                                           attr_translate=self.toolchain.attr_translate,
                                           **kwargs)

    def build(self, *args, **kwargs):
        return self.toolchain.build(self, *args, **kwargs)

    def add_period_constraint(self, clk, period):
        if hasattr(clk, "p"):
            clk = clk.p
        self.toolchain.add_period_constraint(self, clk, period)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_ios = [
    ("clk0", 0, Pins("N9"), IOStandard("LVCMOS18")),
    ("fpga_reset", 0, Pins("T9"), IOStandard("LVCMOS18"), Drive("8")),
    ("fpga_initb", 0, Pins("T12"), IOStandard("LVCMOS18"), Drive("8")),
    ("weim", 0,
        Subsignal("cs4_dtack", Pins("R3"), IOStandard("LVCMOS18"), Drive("8")),
        Subsignal("cs5n", Pins("P10"), IOStandard("LVCMOS18")),
        Subsignal("eb0n", Pins("P9"), IOStandard("LVCMOS18")),
        Subsignal("oen", Pins("R9"), IOStandard("LVCMOS18")),
        Subsignal("data",
                  Pins("T5 T6 P7 N8 P12 T13 R13 T14 P5 N6 T3 T11 T4 R5 M10 T10"),
                  IOStandard("LVCMOS18"), Drive("8")),
        Subsignal("addr",
                  Pins("N5 L7 M7 M8 L8 L9 L10 M11 P11 N11 N12 P13"),
                  IOStandard("LVCMOS18"))
     )
]

_connectors = [
   ("J2",
        "None",  # no 0 pin
        "None",  # 1 +3v3
        "None",  # 2 +3v3
        "None",  # 3 GND
        "None",  # 4 GND
        "None",  # 5 DP USB_OTG_PHY +3V3
        "None",  # 6 DM USB_OTG_PHY +3V3
        "None",  # 7 VBUS USB_OTG_ PHY +3V3
        "None",  # 8 PSW_N USB_OTG_PHY +3V3
        "None",  # 9 ID USB_OTG_PHY +3V3
        "None",  # 10 FAULT USB_OTG_PHY +3V3
        "None",  # 11 RXP Ethernet_PHY +3V3
        "None",  # 12 RXN Ethernet_PHY +3V3
        "None",  # 13 ETH_LINK Ethernet_PHY +2V8
        "None",  # 14 PC_VS2 PC +2V8 PF13
        "None",  # 15 PC_VS1 PC +2V8 PF14
        "None",  # 16 PC_PWRON PC +2V8 PF16
        "None",  # 17 PC_READY PC +2V8 PF17
        "None",  # 18 PWM0 PWM0 +2V8 PE5
        "None",  # 19 TOUT GPT +2V8 PC14
        "None",  # 20 GND POWER
        "None",  # 21 VCC01 (IN) BANK1 SUPPLY VCCO1
        "C16",  # 22 IO_L24P_1 FPGA_BANK1 VCC01
        "C15",  # 23 IO_L24N_1 FPGA_BANK1 VCC01
        "D16",  # 24 IO_L22_P1 FPGA_BANK1 VCC01
        "None",  # 25 GND POWER
        "B14",  # 26 IO_L02N_0 FPGA_BANK0 VCCO0
        "B15",  # 27 IO_L02P_0 FPGA_BANK0
        "A13",  # 28 IO_L04N_0 FPGA_BANK0
        "A14",  # 29 IO_L04P_0 FPGA_BANK0 VCCO0
        "D11",  # 30 IO_L03N_0 FPGA_BANK0 VCCO0
        "C12",  # 31 IO_L03P_0 FPGA_BANK0 VCCO0
        "A10",  # 32 IO_L08N_0 FPGA_BANK0 VCCO0
        "B10",  # 33 IO_L08P_0 FPGA_BANK0 VCCO0
        "A9",  # 34 IO_L10N_0 / GLCK7 FPGA_BANK0 VCCO0
        "C9",  # 35 IO_L10P_0 / GCLK6 FPGA_BANK0 VCCO0
        "B8",  # 36 IO_L12N_0 / GCLK11 FPGA_BANK0 VCCO0
        "A8",  # 37 IO_L12P_0 / GCLK10 FPGA_BANK0 VCCO0
        "B6",  # 38 IO_L15N_0 FPGA_BANK0 VCCO0
        "A6",  # 39 IO_L15P_0 FPGA_BANK0 VCCO0
        "B4",  # 40 IO_L18N_0 FPGA_BANK0 VCCO0
        "A4",  # 41 IO_L18P_0 FPGA_BANK0 VCCO0
        "None",  # 42 GND POWER
        "N3",  # 43 IO_L24P_3 FPGA_BANK3 VCCO3
        "R1",  # 44 IO_L23P_3 FPGA_BANK3 VCCO3
        "P1",  # 45 IO_L22N_3 FPGA_BANK3 VCCO3
        "N1",  # 46 IO_L20N_3 FPGA_BANK3 VCCO3
        "M1",  # 47 IO_L20P_3 FPGA_BANK3 VCCO3
        "H3",  # 48 IO_L12P_3 FPGA_BANK3 VCCO3
        "K1",  # 49 IO_L15N_3 FPGA_BANK3 VCCO3
        "J1",  # 50 IO_L14N_3 FPGA_BANK3 VCCO3
        "H1",  # 51 IO_L11N_3 FPGA_BANK3 VCCO3
        "G1",  # 52 IO_L08N_3 FPGA_BANK3 VCCO3
        "F1",  # 53 IO_L08P_3 FPGA_BANK3 VCCO3
        "E1",  # 54 IO_L03N_3 FPGA_BANK3 VCCO3
        "D1",  # 55 IO_LO3P_3 FPGA_BANK3 VCCO3
        "C1",  # 56 IO_L01N_3 FPGA_BANK3 VCCO3
        "None",  # 57 GND POWER
        "None",  # 58 TRSTN JTAG +2V8
        "None",  # 59 TDI JTAG +2V8
        "None",  # 60 TCK JTAG +2V8
        "None",  # 61 TDO JTAG +2V8
        "None",  # 62 TMS JTAG +2V8
        "None",  # 63 GND POWER
        "C2",  # 64 IO_L01P_3 FPGA_BANK3 VCCO3
        "D3",  # 65 IO_L02N_3 FPGA_BANK3 VCCO3
        "D4",  # 66 IO_L02P_3 FPGA_BANK3 VCCO3
        "F4",  # 67 IP_LO4N_3 FPGA_BANK3 VCCO3
        "G2",  # 68 IO_L11P_3 FPGA_BANK3 VCCO3
        "J2",  # 69 IO_L14P_3 FPGA_BANK3 VCCO3
        "K3",  # 70 IO_L15P_3 FPGA_BANK3 VCCO3
        "J3",  # 71 IO_L12N_3 FPGA_BANK3 VCCO3
        "N2",  # 72 IO_L22P_3 FPGA_BANK3 VCCO3
        "P2",  # 73 IO_L23N_3 FPGA_BANK3 VCCO3
        "M4",  # 74 IO_L24N_3 FPGA_BANK3 VCCO3
        "L6",  # 75 IP_L25N_3 FPGA_BANK3 VCCO3
        "None",  # 76 VCCO3 (IN) BANK3 SUPPLY VCCO3 (3.3Vmax)
        "None",  # 77 VCCO3 (IN) BANK3 SUPPLY VCCO3 (3.3Vmax)
        "A3",  # 78 IO_L19P_0 FPGA_BANK0 VCCO0
        "B3",  # 79 IO_L19N_0 FPGA_BANK0 VCCO0
        "A5",  # 80 IO_L17P_0 FPGA_BANK0 VCCO0
        "C5",  # 81 IO_L17N_0 FPGA_BANK0 VCCO0
        "D7",  # 82 IO_L16P_0 FPGA_BANK0 VCCO0
        "C6",  # 83 IO_L16N_0 FPGA_BANK0 VCCO0
        "C8",  # 84 IO_L11P_0 / GCLK8 FPGA_BANK0 VCCO0
        "D8",  # 85 IO_L11N_0 / GCLK9 FPGA_BANK0 VCCO0
        "C10",  # 86 IO_L09P_0 / GCLK4 FPGA_BANK0 VCCO0
        "D9",  # 87 IO_L09N_0 / GCLK5 FPGA_BANK0 VCCO0
        "C11",  # 88 IO_L07P_0 FPGA_BANK0 VCCO0
        "A11",  # 89 IO_L07N_0 FPGA_BANK0 VCCO0
        "D13",  # 90 IO_L01P_0 FPGA_BANK0 VCCO0
        "C13",  # 91 IO_L01N_0 FPGA_BANK0 VCCO0
        "None",  # 92 VCCO0 (IN) BANK0 SUPPLY VCCO0 (3.3Vmax)
        "None",  # 93 VCCO0 (IN) BANK0 SUPPLY VCCO0 (3.3Vmax)
        "None",  # 94 GND POWER VCCO0 A13
        "D15",  # 95 IO_L22N_1 FPGA_BANK1 VCC01
        "E13",  # 96 IO_L23P_1 FPGA_BANK1 VCC01
        "D14",  # 97 IO_L23N_1 FPGA_BANK1 VCC01
        "E14",  # 98 IO_L20P_1 FPGA_BANK1 VCC01
        "F13",  # 99 IO_L20N_1 FPGA_BANK1 VCC01
        "None",  # 100 GND POWER (3.3Vmax)
        "None",  # 101 USR_RESETN (open CONFIG Pos PC15 +2V8 drain with pullup)
        "None",  # 102 TIN GPT +2V8
        "None",  # 103 EXTAL_26M CONFIG +2V5
        "None",  # 104 RX3 RS232_3 RS232
        "None",  # 105 TX3 RS232_3 RS232
        "None",  # 106 RX1 RS232_1 RS232
        "None",  # 107 TX1 RS232_1 RS232
        "None",  # 108 BOOT CONFIG +2V8
        "None",  # 109 TXN Ethernet_PHY +3V3
        "None",  # 110 TXP Ethernet_PHY +3V3
        "None",  # 111 ETH_ACTIVITY Ethernet_PHY +2V8
        "None",  # 112 USBH2_NXT USB_HOST2 +2V5 PA3
        "None",  # 113 USBH2_DIR USB_HOST2 +2V5 PA1
        "None",  # 114 USBH2_DATA7 USB_HOST2 +2V5 PA2
        "None",  # 115 USBH2_STP USB_HOST2 +2V5 PA4
        "None")  # 116 USBH2_CLK USB_HOST2 +2V5 PA0
]


class Platform(XilinxPlatform):
    default_clk_name = "clk0"
    default_clk_period = 10

    def __init__(self):
        XilinxPlatform.__init__(self, "xc3s200a-ft256-4", _ios, _connectors)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("user_led", 0, Pins("Y3")),
    ("user_led", 1, Pins("Y1")),
    ("user_led", 2, Pins("W2")),
    ("user_led", 3, Pins("W1")),
    ("user_led", 4, Pins("V3")),
    ("user_led", 5, Pins("V1")),
    ("user_led", 6, Pins("U2")),
    ("user_led", 7, Pins("U1")),

    ("clk100", 0,
        Subsignal("p", Pins("B14"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")),
        Subsignal("n", Pins("A14"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE"))
    ),

    ("gpio", 0, Pins("R8")),

    ("gpmc", 0,
        Subsignal("clk", Pins("R26")),
        Subsignal("a", Pins("N17 N18 L23 L24 N19 N20 N21 N22 P17 P19")),
        Subsignal("d", Pins("N23 N24 R18 R19 P21 P22 R20 R21 P24 P26 R23 R24 T22 T23 U23 R25")),
        Subsignal("we_n", Pins("W26")),
        Subsignal("oe_n", Pins("AA25")),
        Subsignal("ale_n", Pins("AA26")),
        Subsignal("wait", Pins("AD26")),  # WAIT1/BUSY0
        IOStandard("LVCMOS33")),
    # Warning: CS are numbered 1-7 on ARM side and 0-6 on FPGA side.
    # Numbers here are given on the FPGA side.
    ("gpmc_ce_n", 0, Pins("V23"), IOStandard("LVCMOS33")),  # nCS0
    ("gpmc_ce_n", 1, Pins("U25"), IOStandard("LVCMOS33")),  # nCS1
    ("gpmc_ce_n", 2, Pins("W25"), IOStandard("LVCMOS33")),  # nCS6
    ("gpmc_dmareq_n", 0, Pins("T24"), IOStandard("LVCMOS33")),  # nCS2
    ("gpmc_dmareq_n", 1, Pins("T26"), IOStandard("LVCMOS33")),  # nCS3
    ("gpmc_dmareq_n", 2, Pins("V24"), IOStandard("LVCMOS33")),  # nCS4
    ("gpmc_dmareq_n", 3, Pins("V26"), IOStandard("LVCMOS33")),  # nCS5

    # FMC150
    ("fmc150_ctrl", 0,
        Subsignal("spi_sclk", Pins("AE5")),
        Subsignal("spi_data", Pins("AF5")),

        Subsignal("adc_sdo", Pins("U13")),
        Subsignal("adc_en_n", Pins("AA15")),
        Subsignal("adc_reset", Pins("V13")),

        Subsignal("cdce_sdo", Pins("AA8")),
        Subsignal("cdce_en_n", Pins("Y9")),
        Subsignal("cdce_reset_n", Pins("AB7")),
        Subsignal("cdce_pd_n", Pins("AC6")),
        Subsignal("cdce_pll_status", Pins("W7")),
        Subsignal("cdce_ref_en", Pins("W8")),

        Subsignal("dac_sdo", Pins("W9")),
        Subsignal("dac_en_n", Pins("W10")),

        Subsignal("mon_sdo", Pins("AC5")),
        Subsignal("mon_en_n", Pins("AD6")),
        Subsignal("mon_reset_n", Pins("AF6")),
        Subsignal("mon_int_n", Pins("AD5")),

        Subsignal("pg_c2m", Pins("AA23"), IOStandard("LVCMOS33"))
    ),
    ("ti_dac", 0,  # DAC3283
        Subsignal("dat_p", Pins("AA10 AA9 V11 Y11 W14 Y12 AD14 AE13"), IOStandard("LVDS_25")),
        Subsignal("dat_n", Pins("AB11 AB9 V10 AA11 Y13 AA12 AF14 AF13"), IOStandard("LVDS_25")),
        Subsignal("frame_p", Pins("AB13"), IOStandard("LVDS_25")),
        Subsignal("frame_n", Pins("AA13"), IOStandard("LVDS_25")),
        Subsignal("txenable", Pins("AB15"), IOStandard("LVCMOS25"))
    ),
    ("ti_adc", 0,  # ADS62P49
        Subsignal("dat_a_p", Pins("AB14 Y21 W20 AB22 V18 W17 AA21")),
        Subsignal("dat_a_n", Pins("AC14 AA22 Y20 AC22 W19 W18 AB21")),
        Subsignal("dat_b_p", Pins("Y17 U15 AA19 W16 AA18 Y15 V14")),
        Subsignal("dat_b_n", Pins("AA17 V16 AB19 Y16 AB17 AA16 V15")),
        IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")
    ),
    ("fmc150_clocks", 0,
        Subsignal("dac_clk_p", Pins("V12"), IOStandard("LVDS_25")),
        Subsignal("dac_clk_n", Pins("W12"), IOStandard("LVDS_25")),
        Subsignal("adc_clk_p", Pins("AE15"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")),
        Subsignal("adc_clk_n", Pins("AF15"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")),
        Subsignal("clk_to_fpga", Pins("W24"), IOStandard("LVCMOS25"))
    ),

    ("fmc150_ext_trigger", 0, Pins("U26")),

    # Vermeer radar testbed
    # Switch controller
    ("pca9555", 0,
        Subsignal("sda", Pins("C13")),
        Subsignal("scl", Pins("G8")),
        IOStandard("LVCMOS33")
    ),
    # TX path
    ("pe43602", 0,
        Subsignal("d", Pins("H8")),
        Subsignal("clk", Pins("B3")),
        Subsignal("le", Pins("F7")),
        IOStandard("LVCMOS33")
    ),
    ("rfmd2081", 0,
        Subsignal("enx", Pins("E5")),
        Subsignal("sclk", Pins("G6")),
        Subsignal("sdata", Pins("F5")),
        Subsignal("locked", Pins("E6")),
        IOStandard("LVCMOS33")
    ),
    # RX path
    ("lmh6521", 0,
        Subsignal("scsb", Pins("C5")),
        Subsignal("sclk", Pins("G10")),
        Subsignal("sdi", Pins("D5")),
        Subsignal("sdo", Pins("F9")),
        IOStandard("LVCMOS33")
    ),
    ("lmh6521", 1,
        Subsignal("scsb", Pins("E10")),
        Subsignal("sclk", Pins("A4")),
        Subsignal("sdi", Pins("B4")),
        Subsignal("sdo", Pins("H10")),
        IOStandard("LVCMOS33")
    ),
    ("rffc5071", 0,
        Subsignal("enx", Pins("A2")),
        Subsignal("sclk", Pins("G9")),
        Subsignal("sdata", Pins("H9")),
        Subsignal("locked", Pins("A3")),
        IOStandard("LVCMOS33")
    )
]


class Platform(XilinxPlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx150t-fgg676-3", _io)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
        ("clk_fx", 0, Pins("L22"), IOStandard("LVCMOS33")),
        ("clk_if", 0, Pins("K20"), IOStandard("LVCMOS33")),
        ("rst", 0, Pins("A18")),
        # PROG_B and DONE: AA1 U16

        ("fx2", 0,
            Subsignal("sloe", Pins("U15"), Drive(12)),  # M1
            Subsignal("slrd", Pins("N22"), Drive(12)),
            Subsignal("slwr", Pins("M22"), Drive(12)),
            Subsignal("pktend", Pins("AB5"), Drive(12)),  # CSO
            Subsignal("fifoadr", Pins("W17 Y18"), Drive(12)),  # CCLK M0
            Subsignal("cont", Pins("G20")),
            Subsignal("fd", Pins("Y17 V13 W13 AA8 AB8 W6 Y6 Y9 "
                "V21 V22 U20 U22 R20 R22 P18 P19")),
            Subsignal("flag", Pins("F20 F19 F18 AB17")),  # - - - CSI/MOSI
            Subsignal("rdy25", Pins("M21 K21 K22 J21")),
            Subsignal("ctl35", Pins("D19 E20 N20")),
            Subsignal("int45", Pins("C18 V17")),
            Subsignal("pc", Pins("G20 T10 V5 AB9 G19 H20 H19 H18")),
            # - DOUT/BUSY INIT_B RDWR_B DO CS CLK DI
            IOStandard("LVCMOS33")),

        ("mm", 0,
            Subsignal("a", Pins("M20 M19 M18 N19 T19 T21 T22 R19 ",
                        "P20 P21 P22 J22 H21 H22 G22 F21")),
            Subsignal("d", Pins("D20 C20 C19 B21 B20 J19 K19 L19"), Drive(2)),
            Subsignal("wr_n", Pins("C22")),
            Subsignal("rd_n", Pins("D21")),
            Subsignal("psen_n", Pins("D22")),
            IOStandard("LVCMOS33")),

        ("serial", 0,
            Subsignal("tx", Pins("B22"), Misc("SLEW=QUIETIO")),
            Subsignal("rx", Pins("A21"), Misc("PULLDOWN")),
            IOStandard("LVCMOS33")),

        ("ddram_clock", 0,
            Subsignal("p", Pins("F2"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("n", Pins("F1"), Misc("OUT_TERM=UNTUNED_50")),
            IOStandard("SSTL18_II")),

        ("ddram", 0,
            Subsignal("dqs", Pins("L3 T2"), IOStandard("SSTL18_II"),  # DIFF_
                    Misc("IN_TERM=NONE")),
            Subsignal("dqs_n", Pins("L1 T1"), IOStandard("SSTL18_II"),  # DIFF_
                    Misc("IN_TERM=NONE")),
            Subsignal("dm", Pins("H1 H2"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("dq", Pins("M1 M2 J1 K2 J3 K1 N3 N1 "
                    "U1 U3 P1 R3 P2 R1 V2 V1"), Misc("IN_TERM=NONE")),
            Subsignal("ras_n", Pins("N4"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("cas_n", Pins("P3"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("a", Pins("M5 K6 B1 J4 L4 K3 M4 K5 G3 G1 K4 C3 C1"),
                    Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("ba", Pins("E3 E1 D1"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("cke", Pins("J6"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("cs_n", Pins("H6")),  # NC!
            Subsignal("odt", Pins("M3"), Misc("OUT_TERM=UNTUNED_50")),
            Subsignal("we_n", Pins("D2")),
            Subsignal("rzq", Pins("AA2")),
            Subsignal("zio", Pins("Y2")),
            IOStandard("SSTL18_II")),

        ("i2c", 0,
            Subsignal("scl", Pins("F22")),
            Subsignal("sda", Pins("E22")),
            IOStandard("LVCMOS33")),

        ("sd", 0,
            Subsignal("sck", Pins("H11")),
            Subsignal("d3", Pins("H14")),
            Subsignal("d", Pins("P10")),
            Subsignal("d1", Pins("T18")),
            Subsignal("d2", Pins("R17")),
            Subsignal("cmd", Pins("H13")),
            IOStandard("LVCMOS33")),

]


class Platform(XilinxPlatform):
    default_clk_name = "clk_if"
    default_clk_period = 20

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx150-3csg484", _io)
        self.add_platform_command("""
CONFIG VCCAUX = "2.5";
""")

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)

        try:
            clk_if = self.lookup_request("clk_if")
            clk_fx = self.lookup_request("clk_fx")
            self.add_platform_command("""
NET "{clk_if}" TNM_NET = "GRPclk_if";
NET "{clk_fx}" TNM_NET = "GRPclk_fx";
TIMESPEC "TSclk_fx" = PERIOD "GRPclk_fx" 20.83333 ns HIGH 50%;
TIMESPEC "TSclk_if" = PERIOD "GRPclk_if" 20 ns HIGH 50%;
TIMESPEC "TSclk_fx2if" = FROM "GRPclk_fx" TO "GRPclk_if" 3 ns DATAPATHONLY;
TIMESPEC "TSclk_if2fx" = FROM "GRPclk_if" TO "GRPclk_fx" 3 ns DATAPATHONLY;
""", clk_if=clk_if, clk_fx=clk_fx)
        except ConstraintError:
            pass



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import TinyProgProgrammer

_io = [
    ("user_led", 0, Pins("B3"), IOStandard("LVCMOS33")),

    ("usb", 0,
        Subsignal("d_p", Pins("B4")),
        Subsignal("d_n", Pins("A4")),
        Subsignal("pullup", Pins("A3")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("F7"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("G7"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("G6"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("H7"), IOStandard("LVCMOS33")),
        Subsignal("wp", Pins("H4"), IOStandard("LVCMOS33")),
        Subsignal("hold", Pins("J8"), IOStandard("LVCMOS33"))
    ),

    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("F7"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("G7"), IOStandard("LVCMOS33")),
        Subsignal("dq", Pins("G6 H7 H4 J8"), IOStandard("LVCMOS33"))
    ),

    ("clk16", 0, Pins("B2"), IOStandard("LVCMOS33"))
]

_connectors = [
    # A2-H2, Pins 1-13
    # H9-A6, Pins 14-24
    # G1-J2, Pins 25-31
    ("GPIO", "A2 A1 B1 C2 C1 D2 D1 E2 E1 G2 H1 J1 H2 H9 D9 D8 C9 A9 B8 A8 B7 A7 B6 A6"),
    ("EXTRA", "G1 J3 J4 G9 J9 E8 J2")
]


# Default peripherals
serial = [
    ("serial", 0,
        Subsignal("tx", Pins("GPIO:0")),
        Subsignal("rx", Pins("GPIO:1")),
        IOStandard("LVCMOS33")
    )
]


class Platform(LatticePlatform):
    default_clk_name = "clk16"
    default_clk_period = 62.5

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-lp8k-cm81", _io, _connectors,
                                 toolchain="icestorm")

    def create_programmer(self):
        return TinyProgProgrammer()



from migen.fhdl.structure import *
from migen.fhdl.module import Module
from migen.fhdl.bitcontainer import bits_for


def split(v, *counts):
    r = []
    offset = 0
    for n in counts:
        if n != 0:
            r.append(v[offset:offset+n])
        else:
            r.append(None)
        offset += n
    return tuple(r)


def displacer(signal, shift, output, n=None, reverse=False):
    if shift is None:
        return output.eq(signal)
    if n is None:
        n = 2**len(shift)
    w = len(signal)
    if reverse:
        r = reversed(range(n))
    else:
        r = range(n)
    l = [Replicate(shift == i, w) & signal for i in r]
    return output.eq(Cat(*l))


def chooser(signal, shift, output, n=None, reverse=False):
    if shift is None:
        return output.eq(signal)
    if n is None:
        n = 2**len(shift)
    w = len(output)
    cases = {}
    for i in range(n):
        if reverse:
            s = n - i - 1
        else:
            s = i
        cases[i] = [output.eq(signal[s*w:(s+1)*w])]
    return Case(shift, cases).makedefault()


def timeline(trigger, events):
    lastevent = max([e[0] for e in events])
    counter = Signal(max=lastevent+1)

    counterlogic = If(counter != 0,
        counter.eq(counter + 1)
    ).Elif(trigger,
        counter.eq(1)
    )
    # insert counter reset if it doesn't naturally overflow
    # (test if lastevent+1 is a power of 2)
    if (lastevent & (lastevent + 1)) != 0:
        counterlogic = If(counter == lastevent,
            counter.eq(0)
        ).Else(
            counterlogic
        )

    def get_cond(e):
        if e[0] == 0:
            return trigger & (counter == 0)
        else:
            return counter == e[0]
    sync = [If(get_cond(e), *e[1]) for e in events]
    sync.append(counterlogic)
    return sync


class WaitTimer(Module):
    def __init__(self, t):
        self.wait = Signal()
        self.done = Signal()

        # # #

        count = Signal(bits_for(t), reset=t)
        self.comb += self.done.eq(count == 0)
        self.sync += \
            If(self.wait,
                If(~self.done, count.eq(count - 1))
            ).Else(count.eq(count.reset))


class BitSlip(Module):
    def __init__(self, dw):
        self.i = Signal(dw)
        self.o = Signal(dw)
        self.value = Signal(max=dw)

        # # #

        r = Signal(2*dw)
        self.sync += r.eq(Cat(r[dw:], self.i))
        cases = {}
        for i in range(dw):
            cases[i] = self.o.eq(r[i:dw+i])
        self.sync += Case(self.value, cases)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform

_io_v1_0 = [
    ("user_led", 0, Pins("T16"), IOStandard("LVCMOS25")),  # LED_USER1

    ("clk50", 0, Pins("W19"), IOStandard("LVCMOS25")),

    ("serial", 0,
        Subsignal("rx", Pins("N13")),  # FPGA input, schematics TxD_2V5
        Subsignal("tx", Pins("N17")),  # FPGA output, schematics RxD_2V5
        IOStandard("LVCMOS25")
    ),

    ("clk_sel", 0, Pins("F21"), IOStandard("LVCMOS25")),

    ("vusb_present", 0, Pins("M17"), IOStandard("LVCMOS25")),

    ("sfp_ctl", 0,
        Subsignal("mod_def1", Pins("U7")),
        Subsignal("mod_def2", Pins("T3")),
        Subsignal("los", Pins("P15")),
        Subsignal("mod_present", Pins("U16")),
        Subsignal("rate_select", Pins("N15")),
        Subsignal("tx_disable", Pins("R14")),
        Subsignal("tx_fault", Pins("N14")),
        Subsignal("led", Pins("P16")),
        IOStandard("LVCMOS25")
    ),
    ("sfp_ctl", 1,
        Subsignal("mod_def1", Pins("P17")),
        Subsignal("mod_def2", Pins("U18")),
        Subsignal("los", Pins("R18")),
        Subsignal("mod_present", Pins("W20")),
        Subsignal("rate_select", Pins("T18")),
        Subsignal("tx_disable", Pins("R17")),
        Subsignal("tx_fault", Pins("U17")),
        Subsignal("led", Pins("R19")),
        IOStandard("LVCMOS25")
    ),
    ("sfp_ctl", 2,
        Subsignal("mod_def1", Pins("P14")),
        Subsignal("mod_def2", Pins("P20")),
        Subsignal("los", Pins("V22")),
        Subsignal("mod_present", Pins("T21")),
        Subsignal("rate_select", Pins("T20")),
        Subsignal("tx_disable", Pins("U21")),
        Subsignal("tx_fault", Pins("R16")),
        Subsignal("led", Pins("P19")),
        IOStandard("LVCMOS25")
    ),
]


_io_v1_1 = [
    ("user_led", 0, Pins("N17"), IOStandard("LVCMOS25")),  # LED_USER1
    ("user_led", 1, Pins("V22"), IOStandard("LVCMOS25")),  # LED_USER2
    ("user_led", 2, Pins("T21"), IOStandard("LVCMOS25")),  # LED_USER3

    ("serial", 0,
        Subsignal("rx", Pins("M17")),  # FPGA input, schematics TxD_2V5
        Subsignal("tx", Pins("T16")),  # FPGA output, schematics RxD_2V5
        IOStandard("LVCMOS25")
    ),

    ("vusb_present", 0, Pins("N13"), IOStandard("LVCMOS25")),

    ("sfp_ctl", 0,
        Subsignal("los", Pins("N15")),
        Subsignal("mod_present_n", Pins("P16")),
        Subsignal("rate_select", Pins("R14")),
        Subsignal("rate_select1", Pins("P15")),
        Subsignal("tx_disable", Pins("N14")),
        Subsignal("tx_fault", Pins("U7")),
        Subsignal("led", Pins("U16")),
        IOStandard("LVCMOS25")
    ),

    ("sfp_ctl", 1,
        Subsignal("los", Pins("T18")),
        Subsignal("mod_present_n", Pins("P17")),
        Subsignal("rate_select", Pins("U18")),
        Subsignal("rate_select1", Pins("R18")),
        Subsignal("tx_disable", Pins("R17")),
        Subsignal("tx_fault", Pins("U17")),
        Subsignal("led", Pins("R19")),
        IOStandard("LVCMOS25")
    ),

    ("sfp_ctl", 2,
        Subsignal("los", Pins("R16")),
        Subsignal("mod_present_n", Pins("T20")),
        Subsignal("rate_select", Pins("P14")),
        Subsignal("rate_select1", Pins("U21")),
        Subsignal("tx_disable", Pins("P20")),
        Subsignal("tx_fault", Pins("P19")),
        Subsignal("led", Pins("W20")),
        IOStandard("LVCMOS25")
    ),
]


_io_common = [
    ("i2c", 0,
        Subsignal("scl", Pins("J16")),
        Subsignal("sda", Pins("F15")),
        IOStandard("LVCMOS25")
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("T19")),
        Subsignal("dq", Pins("P22 R22 P21 R21")),
        # "clk" is on CCLK
        IOStandard("LVCMOS25")
    ),
    ("spiflash2x", 0,
        Subsignal("cs_n", Pins("T19")),
        Subsignal("dq", Pins("P22 R22")),
        Subsignal("wp", Pins("P21")),
        Subsignal("hold", Pins("R21")),
        # "clk" is on CCLK
        IOStandard("LVCMOS25")
    ),

    ("clk125_gtp", 0,
        Subsignal("p", Pins("F10")),
        Subsignal("n", Pins("E10")),
    ),

    ("si5324_clkin", 0,
        Subsignal("p", Pins("U20")),
        Subsignal("n", Pins("V20")),
        IOStandard("LVDS_25"),
    ),

    ("si5324_clkout", 0,
        Subsignal("p", Pins("F6")),
        Subsignal("n", Pins("E6")),
    ),

    ("si5324_clkout_fabric", 0,
        Subsignal("p", Pins("Y18")),
        Subsignal("n", Pins("Y19")),
        IOStandard("LVDS_25"),
    ),

    ("sfp", 0,
        Subsignal("txp", Pins("B4")),
        Subsignal("txn", Pins("A4")),
        Subsignal("rxp", Pins("B8")),
        Subsignal("rxn", Pins("A8")),
    ),
    ("sfp", 1,
        Subsignal("txp", Pins("D5")),
        Subsignal("txn", Pins("C5")),
        Subsignal("rxp", Pins("D11")),
        Subsignal("rxn", Pins("C11")),
    ),
    ("sfp", 2,
        Subsignal("txp", Pins("B6")),
        Subsignal("txn", Pins("A6")),
        Subsignal("rxp", Pins("B10")),
        Subsignal("rxn", Pins("A10")),
    ),
    ("sata", 0,
        Subsignal("txp", Pins("D7")),
        Subsignal("txn", Pins("C7")),
        Subsignal("rxp", Pins("D9")),
        Subsignal("rxn", Pins("C9")),
    ),

    ("ddram", 0,
        Subsignal("a", Pins(
            "L6 M5 P6 K6 M1 M3 N2 M6 "
            "P1 P2 L4 N5 L3 R1 N3"),
            IOStandard("SSTL15")),
        Subsignal("ba", Pins("L5 M2 N4"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("J4"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("J6"), IOStandard("SSTL15")),
        Subsignal("we_n", Pins("K3"), IOStandard("SSTL15")),
        # Subsignal("cs_n", Pins(""), IOStandard("SSTL15")),
        Subsignal("dm", Pins("G2 E2"), IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "G3 J1 H4 H5 H2 K1 H3 J5 "
            "G1 B1 F1 F3 C2 A1 D2 B2"),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_50")),
        Subsignal("dqs_p", Pins("K2 E1"), IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n", Pins("J2 D1"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p", Pins("P5"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("P4"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke", Pins("L1"), IOStandard("SSTL15")),
        Subsignal("odt", Pins("K4"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("G4"), IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
    ),
]


_connectors = [
    ("eem0", {
        "d0_cc_n": "T4",
        "d0_cc_p": "R4",
        "d1_n": "R2",
        "d1_p": "R3",
        "d2_n": "U1",
        "d2_p": "T1",
        "d3_n": "V2",
        "d3_p": "U2",
        "d4_n": "Y1",
        "d4_p": "W1",
        "d5_n": "Y2",
        "d5_p": "W2",
        "d6_n": "AB1",
        "d6_p": "AA1",
        "d7_n": "AA4",
        "d7_p": "Y4",
    }),

    ("eem1", {
        "d0_cc_n": "U5",
        "d0_cc_p": "T5",
        "d1_n": "W7",
        "d1_p": "V7",
        "d2_n": "V3",
        "d2_p": "U3",
        "d3_n": "V8",
        "d3_p": "V9",
        "d4_n": "AB2",
        "d4_p": "AB3",
        "d5_n": "T6",
        "d5_p": "R6",
        "d6_n": "V5",
        "d6_p": "U6",
        "d7_n": "AB5",
        "d7_p": "AA5",
    }),

    ("eem2", {
        "d0_cc_n": "W4",
        "d0_cc_p": "V4",
        "d1_n": "AA3",
        "d1_p": "Y3",
        "d2_n": "W5",
        "d2_p": "W6",
        "d3_n": "Y7",
        "d3_p": "Y8",
        "d4_n": "AA6",
        "d4_p": "Y6",
        "d5_n": "AB6",
        "d5_p": "AB7",
        "d6_n": "AB8",
        "d6_p": "AA8",
        "d7_n": "Y9",
        "d7_p": "W9",
    }),

    ("eem3", {
        "d0_cc_n": "L20",
        "d0_cc_p": "L19",
        "d1_n": "L13",
        "d1_p": "M13",
        "d2_n": "M20",
        "d2_p": "N20",
        "d3_n": "K14",
        "d3_p": "K13",
        "d4_n": "L15",
        "d4_p": "L14",
        "d5_n": "M16",
        "d5_p": "M15",
        "d6_n": "L18",
        "d6_p": "M18",
        "d7_n": "N19",
        "d7_p": "N18",
    }),

    ("eem4", {
        "d0_cc_n": "V19",
        "d0_cc_p": "V18",
        "d1_n": "Y22",
        "d1_p": "Y21",
        "d2_n": "W17",
        "d2_p": "V17",
        "d3_n": "W22",
        "d3_p": "W21",
        "d4_n": "AB18",
        "d4_p": "AA18",
        "d5_n": "AB20",
        "d5_p": "AA19",
        "d6_n": "AA21",
        "d6_p": "AA20",
        "d7_n": "AB22",
        "d7_p": "AB21",
    }),

    ("eem5", {
        "d0_cc_n": "K19",
        "d0_cc_p": "K18",
        "d1_n": "H15",
        "d1_p": "J15",
        "d2_n": "K22",
        "d2_p": "K21",
        "d3_n": "M22",
        "d3_p": "N22",
        "d4_n": "G18",
        "d4_p": "G17",
        "d5_n": "J21",
        "d5_p": "J20",
        "d6_n": "L21",
        "d6_p": "M21",
        "d7_n": "K16",
        "d7_p": "L16",
    }),

    ("eem6", {
        "d0_cc_n": "H19",
        "d0_cc_p": "J19",
        "d1_n": "H18",
        "d1_p": "H17",
        "d2_n": "G20",
        "d2_p": "H20",
        "d3_n": "J17",
        "d3_p": "K17",
        "d4_n": "H14",
        "d4_p": "J14",
        "d5_n": "H22",
        "d5_p": "J22",
        "d6_n": "G13",
        "d6_p": "H13",
        "d7_n": "G16",
        "d7_p": "G15",
    }),

    ("eem7", {
        "d0_cc_n": "D19",
        "d0_cc_p": "E19",
        "d1_n": "G22",
        "d1_p": "G21",
        "d2_n": "F20",
        "d2_p": "F19",
        "d3_n": "F14",
        "d3_p": "F13",
        "d4_n": "E14",
        "d4_p": "E13",
        "d5_n": "D15",
        "d5_p": "D14",
        "d6_n": "E18",
        "d6_p": "F18",
        "d7_n": "E17",
        "d7_p": "F16",
    }),

    ("eem8", {
        "d0_cc_n": "C17",
        "d0_cc_p": "D17",
        "d1_n": "A16",
        "d1_p": "A15",
        "d2_n": "A14",
        "d2_p": "A13",
        "d3_n": "A19",
        "d3_p": "A18",
        "d4_n": "B13",
        "d4_p": "C13",
        "d5_n": "C15",
        "d5_p": "C14",
        "d6_n": "A20",
        "d6_p": "B20",
        "d7_n": "B16",
        "d7_p": "B15",
    }),

    ("eem9", {
        "d0_cc_n": "C19",
        "d0_cc_p": "C18",
        "d1_n": "A21",
        "d1_p": "B21",
        "d2_n": "D16",
        "d2_p": "E16",
        "d3_n": "B18",
        "d3_p": "B17",
        "d4_n": "C20",
        "d4_p": "D20",
        "d5_n": "D22",
        "d5_p": "E22",
        "d6_n": "B22",
        "d6_p": "C22",
        "d7_n": "D21",
        "d7_p": "E21",
    }),

    ("eem10", {
        "d0_cc_n": "V14",
        "d0_cc_p": "V13",
        "d1_n": "AA16",
        "d1_p": "Y16",
        "d2_n": "AB17",
        "d2_p": "AB16",
        "d3_n": "AB13",
        "d3_p": "AA13",
        "d4_n": "AA14",
        "d4_p": "Y13",
        "d5_n": "AB15",
        "d5_p": "AA15",
        "d6_n": "W16",
        "d6_p": "W15",
        "d7_n": "T15",
        "d7_p": "T14",
    }),

    ("eem11", {
        "d0_cc_n": "W12",
        "d0_cc_p": "W11",
        "d1_n": "V15",
        "d1_p": "U15",
        "d2_n": "Y14",
        "d2_p": "W14",
        "d3_n": "W10",
        "d3_p": "V10",
        "d4_n": "Y12",
        "d4_p": "Y11",
        "d5_n": "AB12",
        "d5_p": "AB11",
        "d6_n": "AA11",
        "d6_p": "AA10",
        "d7_n": "AB10",
        "d7_p": "AA9",
    }),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20.0
    userid = 0xffffffff

    def __init__(self, hw_rev="v1.0"):
        if hw_rev == "v1.0":
            io_rev = _io_v1_0
        elif hw_rev == "v1.1":
            io_rev = _io_v1_1
        else:
            raise ValueError("Unknown hardware revision", hw_rev)

        XilinxPlatform.__init__(
                self, "xc7a100t-fgg484-2", _io_common + io_rev, _connectors,
                toolchain="vivado")
        self.add_platform_command(
                "set_property INTERNAL_VREF 0.750 [get_iobanks 35]")
        self.toolchain.bitstream_commands.extend([
            # NOTE: disable this on Kasli/v1.0 boards where the XADC reference
            # has not been fixed.
            "set_property BITSTREAM.CONFIG.OVERTEMPPOWERDOWN Enable [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS True [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGRATE 33 [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 2 [current_design]",
            "set_property BITSTREAM.CONFIG.USR_ACCESS TIMESTAMP [current_design]",
            "set_property BITSTREAM.CONFIG.USERID \"{:#010x}\" [current_design]".format(self.userid),
            "set_property CFGBVS VCCO [current_design]",
            "set_property CONFIG_VOLTAGE 2.5 [current_design]",
            ])



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    # System clock (Differential 200MHz)
    ("clk200", 0,
        Subsignal("p", Pins("J9"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE")),
        Subsignal("n", Pins("H9"), IOStandard("LVDS_25"), Misc("DIFF_TERM=TRUE"))
    ),

    # User clock (66MHz)
    ("clk66", 0, Pins("U23"), IOStandard("LVCMOS25")),

    # CPU reset switch
    ("cpu_reset", 0, Pins("H10"), IOStandard("SSTL15")),

    # LEDs
    ("user_led", 0, Pins("AC22"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 1, Pins("AC24"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 2, Pins("AE22"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 3, Pins("AE23"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 4, Pins("AB23"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 5, Pins("AG23"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 6, Pins("AE24"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),
    ("user_led", 7, Pins("AD24"), IOStandard("LVCMOS25"), Misc("SLEW=SLOW")),

    # USB-to-UART
    ("serial", 0,
        Subsignal("tx", Pins("J25"), IOStandard("LVCMOS25")),
        Subsignal("rx", Pins("J24"), IOStandard("LVCMOS25"))
    ),

    # 10/100/1000 Tri-Speed Ethernet PHY
    ("eth_clocks", 0,
        Subsignal("rx", Pins("AP11")),
        Subsignal("tx", Pins("AD12")),
        IOStandard("LVCMOS25")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("AH13")),
        Subsignal("rx_dv", Pins("AM13")),
        Subsignal("rx_er", Pins("AG12")),
        Subsignal("rx_data", Pins("AN13 AF14 AE14 AN12 AM12 AD11 AC12 AC13")),
        Subsignal("tx_en", Pins("AJ10")),
        Subsignal("tx_er", Pins("AH10")),
        Subsignal("tx_data", Pins("AM11 AL11 AG10 AG11 AL10 AM10 AE11 AF11")),
        Subsignal("col", Pins("AK13")),
        Subsignal("crs", Pins("AL13")),
        IOStandard("LVCMOS25")
    )
]


class Platform(XilinxPlatform):
    default_clk_name = "clk200"
    default_clk_period = 5

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6vlx240t-ff1156-1", _io)



# This file is Copyright (c) 2017 Serge 'q3k' Bazanski <serge@bazanski.pl>
# License: BSD

from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import LatticeProgrammer


_io = [
    ("clk100", 0, Pins("P3"), IOStandard("LVDS")),
    ("rst_n", 0, Pins("T1"), IOStandard("LVCMOS33")),

    ("user_led", 0, Pins("E16"), IOStandard("LVCMOS25")),
    ("user_led", 1, Pins("D17"), IOStandard("LVCMOS25")),
    ("user_led", 2, Pins("D18"), IOStandard("LVCMOS25")),
    ("user_led", 3, Pins("E18"), IOStandard("LVCMOS25")),
    ("user_led", 4, Pins("F17"), IOStandard("LVCMOS25")),
    ("user_led", 5, Pins("F18"), IOStandard("LVCMOS25")),
    ("user_led", 6, Pins("E17"), IOStandard("LVCMOS25")),
    ("user_led", 7, Pins("F16"), IOStandard("LVCMOS25")),

    ("user_dip_btn", 0, Pins("H2"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 1, Pins("K3"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 2, Pins("G3"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 3, Pins("F2"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 4, Pins("J18"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 5, Pins("K18"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 6, Pins("K19"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 7, Pins("K20"), IOStandard("LVCMOS25")),

    ("serial", 0,
        Subsignal("rx", Pins("C11"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("A11"), IOStandard("LVCMOS33")),
    ),


    ("eth_clocks", 0,
        Subsignal("tx", Pins("P19")),
        Subsignal("rx", Pins("L20")),
        IOStandard("LVCMOS25")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("U17")),
        Subsignal("mdio", Pins("U18")),
        Subsignal("mdc", Pins("T18")),
        Subsignal("rx_ctl", Pins("U19")),
        Subsignal("rx_data", Pins("T20 U20 T19 R18")),
        Subsignal("tx_ctl", Pins("R20")),
        Subsignal("tx_data", Pins("N19 N20 P18 P20")),
        IOStandard("LVCMOS25")
    ),

    ("eth_clocks", 1,
        Subsignal("tx", Pins("C20")),
        Subsignal("rx", Pins("J19")),
        IOStandard("LVCMOS25")
    ),
    ("eth", 1,
        Subsignal("rst_n", Pins("F20")),
        Subsignal("mdio", Pins("H20")),
        Subsignal("mdc", Pins("G19")),
        Subsignal("rx_ctl", Pins("F19")),
        Subsignal("rx_data", Pins("G18 G16 H18 H17")),
        Subsignal("tx_ctl", Pins("E19")),
        Subsignal("tx_data", Pins("J17 J16 D19 D20")),
        IOStandard("LVCMOS25")
    ),

    ("ext_clk", 0,
        Subsignal("p", Pins("A4")),
        Subsignal("n", Pins("A5")),
        IOStandard("LVDS")
    ),

    ("pcie_x1", 0,
        Subsignal("clk_p", Pins("Y11")),
        Subsignal("clk_n", Pins("Y12")),
        Subsignal("rx_p", Pins("Y5")),
        Subsignal("rx_n", Pins("Y6")),
        Subsignal("tx_p", Pins("W4")),
        Subsignal("tx_n", Pins("W5")),
        Subsignal("perst", Pins("A6"), IOStandard("LVCMOS33")),
    ),
]


_connectors = [
   ("X3",
        "None",  # (no pin 0)
        "None",  #  1 GND
        "None",  #  2 N/C
        "None",  #  3 +2V5
        "B19",   #  4 EXPCON_IO29
        "B12",   #  5 EXPCON_IO30
        "B9",    #  6 EXPCON_IO31
        "E6",    #  7 EXPCON_IO32
        "D6",    #  8 EXPCON_IO33
        "E7",    #  9 EXPCON_IO34
        "D7",    # 10 EXPCON_IO35
        "B11",   # 11 EXPCON_IO36
        "B6",    # 12 EXPCON_IO37
        "E9",    # 13 EXPCON_IO38
        "D9",    # 14 EXPCON_IO39
        "B8",    # 15 EXPCON_IO40
        "C8",    # 16 EXPCON_IO41
        "D8",    # 17 EXPCON_IO42
        "E8",    # 18 EXPCON_IO43
        "C7",    # 19 EXPCON_IO44
        "C6",    # 20 EXPCON_IO45
        "None",  # 21 +5V
        "None",  # 22 GND
        "None",  # 23 +2V5
        "None",  # 24 GND
        "None",  # 25 +3V3
        "None",  # 26 GND
        "None",  # 27 +3V3
        "None",  # 28 GND
        "None",  # 29 EXPCON_OSC
        "None",  # 30 GND
        "None",  # 31 EXPCON_CLKIN
        "None",  # 32 GND
        "None",  # 33 EXPCON_CLKOUT
        "None",  # 34 GND
        "None",  # 35 +3V3
        "None",  # 36 GND
        "None",  # 37 +3V3
        "None",  # 38 GND
        "None",  # 39 +3V3
        "None",  # 40 GND
    ),
]


class Platform(LatticePlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self, **kwargs):
        LatticePlatform.__init__(self, "LFE5UM5G-45F-8BG381C", _io, _connectors, **kwargs)

    def do_finalize(self, fragment):
        LatticePlatform.do_finalize(self, fragment)
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks", 0).rx, 8.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks", 1).rx, 8.0)
        except ConstraintError:
            pass

    def create_programmer(self, with_ispclock=True):
        _xcf_ispclock = """
        <Device>
            <SelectedProg value="FALSE"/>
            <Pos>2</Pos>
            <Vendor>Lattice</Vendor>
            <Family>ispCLOCK</Family>
            <Name>ispPAC-CLK5406D</Name>
            <IDCode>0x00191043</IDCode>
            <Operation>Erase,Program,Verify</Operation>
            <Bypass>
                <InstrLen>8</InstrLen>
                <InstrVal>11111111</InstrVal>
                <BScanLen>1</BScanLen>
                <BScanVal>0</BScanVal>
            </Bypass>
        </Device>
"""

        _xcf_template = """
<?xml version='1.0' encoding='utf-8' ?>
<!DOCTYPE        ispXCF    SYSTEM    "IspXCF.dtd" >
<ispXCF version="3.4.1">
    <Comment></Comment>
    <Chain>
        <Comm>JTAG</Comm>
        <Device>
            <SelectedProg value="TRUE"/>
            <Pos>1</Pos>
            <Vendor>Lattice</Vendor>
            <Family>ECP5UM5G</Family>
            <Name>LFE5UM5G-45F</Name>
            <IDCode>0x81112043</IDCode>
            <File>{{bitstream_file}}</File>
            <Operation>Fast Program</Operation>
        </Device>{ispclock}
    </Chain>
    <ProjectOptions>
        <Program>SEQUENTIAL</Program>
        <Process>ENTIRED CHAIN</Process>
        <OperationOverride>No Override</OperationOverride>
        <StartTAP>TLR</StartTAP>
        <EndTAP>TLR</EndTAP>
        <VerifyUsercode value="FALSE"/>
    </ProjectOptions>
    <CableOptions>
        <CableName>USB2</CableName>
        <PortAdd>FTUSB-0</PortAdd>
        <USBID>LATTICE ECP5_5G VERSA BOARD A Location 0000 Serial Lattice ECP5_5G VERSA Board A</USBID>
    </CableOptions>
</ispXCF>
""".format(ispclock=_xcf_ispclock if with_ispclock else "")

        return LatticeProgrammer(_xcf_template)



from migen.fhdl.structure import *
from migen.fhdl.module import Module
from migen.fhdl.specials import Memory, READ_FIRST
from migen.fhdl.bitcontainer import log2_int
from migen.fhdl.decorators import ClockDomainsRenamer
from migen.genlib.cdc import MultiReg, GrayCounter


__all__ = ["SyncFIFO", "SyncFIFOBuffered", "AsyncFIFO", "AsyncFIFOBuffered"]


def _inc(signal, modulo):
    if modulo == 2**len(signal):
        return signal.eq(signal + 1)
    else:
        return If(signal == (modulo - 1),
            signal.eq(0)
        ).Else(
            signal.eq(signal + 1)
        )


class _FIFOInterface:
    """
    Data written to the input interface (`din`, `we`, `writable`) is
    buffered and can be read at the output interface (`dout`, `re`,
    `readable`). The data entry written first to the input
    also appears first on the output.

    Parameters
    ----------
    width : int
        Bit width for the data.
    depth : int
        Depth of the FIFO.

    Attributes
    ----------
    din : in, width
        Input data
    writable : out
        There is space in the FIFO and `we` can be asserted to load new data.
    we : in
        Write enable signal to latch `din` into the FIFO. Does nothing if
        `writable` is not asserted.
    dout : out, width
        Output data. Only valid if `readable` is asserted.
    readable : out
        Output data `dout` valid, FIFO not empty.
    re : in
        Acknowledge `dout`. If asserted, the next entry will be
        available on the next cycle (if `readable` is high then).
    """
    def __init__(self, width, depth):
        self.we = Signal()
        self.writable = Signal()  # not full
        self.re = Signal()
        self.readable = Signal()  # not empty

        self.din = Signal(width, reset_less=True)
        self.dout = Signal(width, reset_less=True)
        self.width = width
        self.depth = depth

    def read(self):
        """Read method for simulation."""
        value = (yield self.dout)
        yield self.re.eq(1)
        yield
        yield self.re.eq(0)
        yield
        return value

    def write(self, data):
        """Write method for simulation."""
        yield self.din.eq(data)
        yield self.we.eq(1)
        yield
        yield self.we.eq(0)
        yield


class SyncFIFO(Module, _FIFOInterface):
    """Synchronous FIFO (first in, first out)

    Read and write interfaces are accessed from the same clock domain.
    If different clock domains are needed, use :class:`AsyncFIFO`.

    {interface}
    level : out
        Number of unread entries.
    replace : in
        Replaces the last entry written into the FIFO with `din`. Does nothing
        if that entry has already been read (i.e. the FIFO is empty).
        Assert in conjunction with `we`.
    """
    __doc__ = __doc__.format(interface=_FIFOInterface.__doc__)

    def __init__(self, width, depth, fwft=True):
        _FIFOInterface.__init__(self, width, depth)

        self.level = Signal(max=depth+1)
        self.replace = Signal()

        ###

        produce = Signal(max=depth)
        consume = Signal(max=depth)
        storage = Memory(self.width, depth)
        self.specials += storage

        wrport = storage.get_port(write_capable=True, mode=READ_FIRST)
        self.specials += wrport
        self.comb += [
            If(self.replace,
                wrport.adr.eq(produce-1)
            ).Else(
                wrport.adr.eq(produce)
            ),
            wrport.dat_w.eq(self.din),
            wrport.we.eq(self.we & (self.writable | self.replace))
        ]
        self.sync += If(self.we & self.writable & ~self.replace,
            _inc(produce, depth))

        do_read = Signal()
        self.comb += do_read.eq(self.readable & self.re)

        rdport = storage.get_port(async_read=fwft, has_re=not fwft, mode=READ_FIRST)
        self.specials += rdport
        self.comb += [
            rdport.adr.eq(consume),
            self.dout.eq(rdport.dat_r)
        ]
        if not fwft:
            self.comb += rdport.re.eq(do_read)
        self.sync += If(do_read, _inc(consume, depth))

        self.sync += \
            If(self.we & self.writable & ~self.replace,
                If(~do_read, self.level.eq(self.level + 1))
            ).Elif(do_read,
                self.level.eq(self.level - 1)
            )
        self.comb += [
            self.writable.eq(self.level != depth),
            self.readable.eq(self.level != 0)
        ]


class SyncFIFOBuffered(Module, _FIFOInterface):
    """Has an interface compatible with SyncFIFO with fwft=True,
    but does not use asynchronous RAM reads that are not compatible
    with block RAMs. Increases latency by one cycle."""
    def __init__(self, width, depth):
        _FIFOInterface.__init__(self, width, depth)
        self.submodules.fifo = fifo = SyncFIFO(width, depth, False)

        self.writable = fifo.writable
        self.din = fifo.din
        self.we = fifo.we
        self.dout = fifo.dout
        self.level = Signal(max=depth+2)

        ###

        self.comb += fifo.re.eq(fifo.readable & (~self.readable | self.re))
        self.sync += \
            If(fifo.re,
                self.readable.eq(1),
            ).Elif(self.re,
                self.readable.eq(0),
            )
        self.comb += self.level.eq(fifo.level + self.readable)


class AsyncFIFO(Module, _FIFOInterface):
    """Asynchronous FIFO (first in, first out)

    Read and write interfaces are accessed from different clock domains,
    named `read` and `write`. Use `ClockDomainsRenamer` to rename to
    other names.

    {interface}
    """
    __doc__ = __doc__.format(interface=_FIFOInterface.__doc__)

    def __init__(self, width, depth):
        _FIFOInterface.__init__(self, width, depth)

        ###

        depth_bits = log2_int(depth, True)

        produce = ClockDomainsRenamer("write")(GrayCounter(depth_bits+1))
        consume = ClockDomainsRenamer("read")(GrayCounter(depth_bits+1))
        self.submodules += produce, consume
        self.comb += [
            produce.ce.eq(self.writable & self.we),
            consume.ce.eq(self.readable & self.re)
        ]

        produce_rdomain = Signal(depth_bits+1)
        produce.q.attr.add("no_retiming")
        self.specials += MultiReg(produce.q, produce_rdomain, "read")
        consume_wdomain = Signal(depth_bits+1)
        consume.q.attr.add("no_retiming")
        self.specials += MultiReg(consume.q, consume_wdomain, "write")
        if depth_bits == 1:
            self.comb += self.writable.eq((produce.q[-1] == consume_wdomain[-1])
                | (produce.q[-2] == consume_wdomain[-2]))
        else:
            self.comb += [
                self.writable.eq((produce.q[-1] == consume_wdomain[-1])
                | (produce.q[-2] == consume_wdomain[-2])
                | (produce.q[:-2] != consume_wdomain[:-2]))
            ]
        self.comb += self.readable.eq(consume.q != produce_rdomain)

        storage = Memory(self.width, depth)
        self.specials += storage
        wrport = storage.get_port(write_capable=True, clock_domain="write")
        self.specials += wrport
        self.comb += [
            wrport.adr.eq(produce.q_binary[:-1]),
            wrport.dat_w.eq(self.din),
            wrport.we.eq(produce.ce)
        ]
        rdport = storage.get_port(clock_domain="read")
        self.specials += rdport
        self.comb += [
            rdport.adr.eq(consume.q_next_binary[:-1]),
            self.dout.eq(rdport.dat_r)
        ]


class AsyncFIFOBuffered(Module, _FIFOInterface):
    """Improves timing when it breaks due to sluggish clock-to-output
    delay in e.g. Xilinx block RAMs. Increases latency by one cycle."""
    def __init__(self, width, depth):
        _FIFOInterface.__init__(self, width, depth)
        self.submodules.fifo = fifo = AsyncFIFO(width, depth)

        self.writable = fifo.writable
        self.din = fifo.din
        self.we = fifo.we

        self.sync.read += \
            If(self.re | ~self.readable,
                self.dout.eq(fifo.dout),
                self.readable.eq(fifo.readable)
            )
        self.comb += fifo.re.eq(self.re | ~self.readable)



import unittest
import os.path
import sys
import subprocess


def _make_test_method(name, foldername):
    def test_method(self):
        filename = name + ".py"
        example_path = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "..", "..", "examples"))
        filepath = os.path.join(example_path, foldername, filename)
        subprocess.check_call(
            [sys.executable, filepath],
            stdout=subprocess.DEVNULL
        )

    return test_method


class TestExamplesSim(unittest.TestCase):
    pass

for name in ("basic1",
              "basic2",
              # skip "fir" as it depends on SciPy
              # "fir",
              "memory"):
    setattr(TestExamplesSim, "test_" + name,
            _make_test_method(name, "sim"))


class TestExamplesBasic(unittest.TestCase):
    pass

for name in ("arrays",
              "fsm",
              "graycounter",
              "local_cd",
              "memory",
              "namer",
              "psync",
              "record",
              "reslice",
              "tristate",
              "two_dividers"):
    setattr(TestExamplesBasic, "test_" + name,
            _make_test_method(name, "basic"))




import os, sys
from migen.build import tools


class GenericProgrammer:
    def __init__(self, flash_proxy_basename=None):
        self.flash_proxy_basename = flash_proxy_basename
        self.flash_proxy_dirs = [
            "~/.migen", "/usr/local/share/migen", "/usr/share/migen",
            "~/.mlabs", "/usr/local/share/mlabs", "/usr/share/mlabs"]

    def set_flash_proxy_dir(self, flash_proxy_dir):
        if flash_proxy_dir is not None:
            self.flash_proxy_dirs = [flash_proxy_dir]

    def find_flash_proxy(self):
        for d in self.flash_proxy_dirs:
            fulldir = os.path.abspath(os.path.expanduser(d))
            fullname = tools.cygpath(os.path.join(fulldir, self.flash_proxy_basename))
            if os.path.exists(fullname):
                return fullname
        raise OSError("Failed to find flash proxy bitstream")

    # must be overloaded by specific programmer
    def load_bitstream(self, bitstream_file):
        raise NotImplementedError

    # must be overloaded by specific programmer
    def flash(self, address, data_file):
        raise NotImplementedError



# This file is Copyright (c) 2018 Florent Kermarrec <florent@enjoy-digital.fr>
# This file is Copyright (c) 2018 William D. Jones <thor0505@comcast.net>
# License: BSD

import os
import subprocess
import sys

from migen.fhdl.structure import _Fragment

from migen.build.generic_platform import *
from migen.build import tools
from migen.build.lattice import common

# TODO:
# - check/document attr_translate.

nextpnr_ecp5_architectures = {
    "lfe5u-25f": "25k",
    "lfe5u-45f": "45k",
    "lfe5u-85f": "85k",
    "lfe5um-25f": "um-25k",
    "lfe5um-45f": "um-45k",
    "lfe5um-85f": "um-85k",
    "lfe5um5g-25f": "um5g-25k",
    "lfe5um5g-45f": "um5g-45k",
    "lfe5um5g-85f": "um5g-85k",
}


def _format_constraint(c):
    if isinstance(c, Pins):
        return ("LOCATE COMP ", " SITE " + "\"" + c.identifiers[0] + "\"")
    elif isinstance(c, IOStandard):
        return ("IOBUF PORT ", " IO_TYPE=" + c.name)
    elif isinstance(c, Misc):
        return ("IOBUF PORT ", " " + c.misc)


def _format_lpf(signame, pin, others, resname):
    fmt_c = [_format_constraint(c) for c in ([Pins(pin)] + others)]
    r = ""
    for pre, suf in fmt_c:
        r += pre + "\"" + signame + "\"" + suf + ";\n"
    return r


def _build_lpf(named_sc, named_pc):
    r = "BLOCK RESETPATHS;\n"
    r += "BLOCK ASYNCPATHS;\n"
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for i, p in enumerate(pins):
                r += _format_lpf(sig + "[" + str(i) + "]", p, others, resname)
        else:
            r += _format_lpf(sig, pins[0], others, resname)
    if named_pc:
        r += "\n" + "\n\n".join(named_pc)
    return r


def _build_script(source, build_template, build_name, architecture,
                  basecfg, freq_constraint):
    if sys.platform in ("win32", "cygwin"):
        script_ext = ".bat"
        build_script_contents = "@echo off\nrem Autogenerated by Migen\n\n"
        fail_stmt = " || exit /b"
    else:
        script_ext = ".sh"
        build_script_contents = "# Autogenerated by Migen\nset -e\n\n"
        fail_stmt = ""

    for s in build_template:
        s_fail = s + "{fail_stmt}\n"  # Required so Windows scripts fail early.
        build_script_contents += s_fail.format(build_name=build_name,
                                               architecture=architecture,
                                               basecfg=basecfg,
                                               freq_constraint=freq_constraint,
                                               fail_stmt=fail_stmt)

    build_script_file = "build_" + build_name + script_ext
    tools.write_to_file(build_script_file, build_script_contents,
                        force_unix=False)
    return build_script_file


def _run_script(script):
    if sys.platform in ("win32", "cygwin"):
        shell = ["cmd", "/c"]
    else:
        shell = ["bash"]

    if subprocess.call(shell + [script]) != 0:
        raise OSError("Subprocess failed")


def yosys_import_sources(platform):
    includes = ""
    reads = []
    for path in platform.verilog_include_paths:
        includes += " -I" + path
    for filename, language, library in platform.sources:
        reads.append("read_{}{} {}".format(
            language, includes, filename))
    return "\n".join(reads)


class LatticeTrellisToolchain:
    attr_translate = {
        # FIXME: document
        "keep": ("keep", "true"),
        "no_retiming": None,
        "async_reg": None,
        "mr_ff": None,
        "mr_false_path": None,
        "ars_ff1": None,
        "ars_ff2": None,
        "ars_false_path": None,
        "no_shreg_extract": None
    }

    special_overrides = common.lattice_ecpx_trellis_special_overrides

    def __init__(self):
        self.yosys_template = [
            "{read_files}",
            "attrmap -tocase keep -imap keep=\"true\" keep=1 -imap keep=\"false\" keep=0 -remove keep=0",
            "synth_ecp5 -nomux -json {build_name}.json -top {build_name}",
        ]

        self.build_template = [
            "yosys -q -l {build_name}.rpt {build_name}.ys",
            "nextpnr-ecp5 --json {build_name}.json --lpf {build_name}.lpf --textcfg {build_name}.config --basecfg {basecfg} --{architecture} --freq {freq_constraint}",
            "ecppack --input {build_name}.config --bit {build_name}.bit --svf {build_name}.svf"
        ]

        self.freq_constraints = dict()

    def build(self, platform, fragment, build_dir="build", build_name="top",
              toolchain_path=None, run=True, **kwargs):
        if toolchain_path is None:
            toolchain_path = "/usr/share/trellis/"
        os.makedirs(build_dir, exist_ok=True)
        cwd = os.getcwd()
        os.chdir(build_dir)

        # generate verilog
        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()
        platform.finalize(fragment)

        top_output = platform.get_verilog(fragment, name=build_name, **kwargs)
        named_sc, named_pc = platform.resolve_signals(top_output.ns)
        top_file = build_name + ".v"
        top_output.write(top_file)
        platform.add_source(top_file)

        # generate constraints
        tools.write_to_file(build_name + ".lpf",
                            _build_lpf(named_sc, named_pc))

        # generate yosys script
        yosys_script_file = build_name + ".ys"
        yosys_script_contents = "\n".join(_.format(build_name=build_name,
                                                   read_files=yosys_import_sources(platform))
                                          for _ in self.yosys_template)
        tools.write_to_file(yosys_script_file, yosys_script_contents)

        # transform platform.device to nextpnr's architecture / basecfg
        (family, size, package) = platform.device.split("-")
        architecture = nextpnr_ecp5_architectures[(family + "-" + size).lower()]
        basecfg = "empty_" + (family + "-" + size).lower() + ".config"
        basecfg = os.path.join(toolchain_path, "misc", "basecfgs", basecfg)
        freq_constraint = str(max(self.freq_constraints.values(),
                                  default=0.0))

        script = _build_script(False, self.build_template, build_name,
                               architecture, basecfg, freq_constraint)

        # run scripts
        if run:
            _run_script(script)

        os.chdir(cwd)

        return top_output.ns

    # Until nextpnr-ecp5 can handle multiple clock domains, use the same
    # approach as the icestorm and use the fastest clock for timing
    # constraints.
    def add_period_constraint(self, platform, clk, period):
        new_freq = 1000.0/period

        if clk not in self.freq_constraints.keys():
            self.freq_constraints[clk] = new_freq
        else:
            raise ConstraintError("Period constraint already added to signal.")



from collections import OrderedDict, namedtuple

from migen.fhdl.structure import *
from migen.fhdl.namer import build_namespace
from migen.fhdl.tools import list_special_ios
from migen.fhdl.structure import _Fragment
from migen.fhdl.conv_output import ConvOutput


_Port = namedtuple("_Port", "name direction")
_Cell = namedtuple("_Cell", "name ports")
_Property = namedtuple("_Property", "name value")
_Instance = namedtuple("_Instance", "name cell properties")
_NetBranch = namedtuple("_NetBranch", "portname instancename")


def _write_cells(cells):
    r = ""
    for cell in cells:
        r += """
        (cell {0.name}
            (cellType GENERIC)
                (view view_1
                    (viewType NETLIST)
                    (interface""".format(cell)
        for port in cell.ports:
            r += """
                        (port {0.name} (direction {0.direction}))""".format(port)
        r += """
                    )
                )
        )"""
    return r


def _write_io(ios):
    r = ""
    for s in ios:
        r += """
                        (port {0.name} (direction {0.direction}))""".format(s)
    return r


def _write_instantiations(instances, cell_library):
    instantiations = ""
    for instance in instances:
        instantiations += """
                        (instance {0.name}
                            (viewRef view_1 (cellRef {0.cell} (libraryRef {1})))""".format(instance, cell_library)
        for prop in instance.properties:
            instantiations += """
                            (property {0} (string "{1}"))""".format(prop.name, prop.value)
        instantiations += """
                        )"""
    return instantiations


def _write_connections(connections):
    r = ""
    for netname, branches in connections.items():
        r += """
                        (net {0}
                            (joined""".format(netname)
        for branch in branches:
            r += """
                                (portRef {0}{1})""".format(branch.portname, "" if branch.instancename == "" else " (instanceRef {})".format(branch.instancename))
        r += """
                            )
                        )"""
    return r


def _write_edif(cells, ios, instances, connections, cell_library, design_name, part, vendor):
    r = """(edif {0}
    (edifVersion 2 0 0)
    (edifLevel 0)
    (keywordMap (keywordLevel 0))
    (external {1}
        (edifLevel 0)
        (technology (numberDefinition))""".format(design_name, cell_library)
    r += _write_cells(cells)
    r += """
    )
    (library {0}_lib
        (edifLevel 0)
        (technology (numberDefinition))
        (cell {0}
            (cellType GENERIC)
                (view view_1
                    (viewType NETLIST)
                    (interface""".format(design_name)
    r += _write_io(ios)
    r += """
                        (designator "{0}")
                    )
                    (contents""".format(part)
    r += _write_instantiations(instances, cell_library)
    r += _write_connections(connections)
    r += """
                    )
                )
        )
    )
    (design {0}
        (cellRef {0} (libraryRef {0}_lib))
        (property PART (string "{1}") (owner "{2}"))
    )
)""".format(design_name, part, vendor)

    return r


def _generate_cells(f):
    cell_dict = OrderedDict()
    for special in f.specials:
        if isinstance(special, Instance):
            port_list = []
            for port in special.items:
                if isinstance(port, Instance.Input):
                    port_list.append(_Port(port.name, "INPUT"))
                elif isinstance(port, Instance.Output):
                    port_list.append(_Port(port.name, "OUTPUT"))
                elif isinstance(port, Instance.InOut):
                    port_list.append(_Port(port.name, "INOUT"))
                elif isinstance(port, Instance.Parameter):
                    pass
                else:
                    raise NotImplementedError("Unsupported instance item")
            if special.of in cell_dict:
                if set(port_list) != set(cell_dict[special.of]):
                    raise ValueError("All instances must have the same ports for EDIF conversion")
            else:
                cell_dict[special.of] = port_list
        else:
            raise ValueError("EDIF conversion can only handle synthesized fragments")
    return [_Cell(k, v) for k, v in cell_dict.items()]


def _generate_instances(f, ns):
    instances = []
    for special in f.specials:
        if isinstance(special, Instance):
            props = []
            for prop in special.items:
                if isinstance(prop, Instance.Input):
                    pass
                elif isinstance(prop, Instance.Output):
                    pass
                elif isinstance(prop, Instance.InOut):
                    pass
                elif isinstance(prop, Instance.Parameter):
                    props.append(_Property(name=prop.name, value=prop.value))
                else:
                    raise NotImplementedError("Unsupported instance item")
            instances.append(_Instance(name=ns.get_name(special), cell=special.of, properties=props))
        else:
            raise ValueError("EDIF conversion can only handle synthesized fragments")
    return instances


def _generate_ios(f, ios, ns):
    outs = list_special_ios(f, False, True, False)
    inouts = list_special_ios(f, False, False, True)
    r = []
    for io in ios:
        direction = "OUTPUT" if io in outs else "INOUT" if io in inouts else "INPUT"
        r.append(_Port(name=ns.get_name(io), direction=direction))
    return r


def _generate_connections(f, ios, ns):
    r = OrderedDict()
    for special in f.specials:
        if isinstance(special, Instance):
            instname = ns.get_name(special)
            for port in special.items:
                if isinstance(port, Instance._IO):
                    s = ns.get_name(port.expr)
                    if s not in r:
                        r[s] = []
                    r[s].append(_NetBranch(portname=port.name, instancename=instname))
                elif isinstance(port, Instance.Parameter):
                    pass
                else:
                    raise NotImplementedError("Unsupported instance item")
        else:
            raise ValueError("EDIF conversion can only handle synthesized fragments")
    for s in ios:
        io = ns.get_name(s)
        if io not in r:
            r[io] = []
        r[io].append(_NetBranch(portname=io, instancename=""))
    return r


def convert(f, ios, cell_library, vendor, device, name="top"):
    if not isinstance(f, _Fragment):
        f = f.get_fragment()
    if f.comb != [] or f.sync != {}:
        raise ValueError("EDIF conversion can only handle synthesized fragments")
    if ios is None:
        ios = set()
    cells = _generate_cells(f)
    ns = build_namespace(list_special_ios(f, True, True, True))
    instances = _generate_instances(f, ns)
    inouts = _generate_ios(f, ios, ns)
    connections = _generate_connections(f, ios, ns)
    src = _write_edif(cells, inouts, instances, connections, cell_library, name, device, vendor)

    r = ConvOutput()
    r.set_main_source(src)
    r.ns = ns
    return r



# This file is Copyright (c) 2015 Matt O'Gorman <mog@rldn.net>
# License: BSD

from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform
from migen.build.xilinx.programmer import XC3SProg, FpgaProg


_io = [
    ("user_led", 0, Pins("P11"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("N9"),  IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("M9"),  IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("P9"),  IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("T8"),  IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("N8"),  IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("P8"),  IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("P7"),  IOStandard("LVCMOS33")),

    ("user_sw", 0, Pins("L1"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("user_sw", 1, Pins("L3"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("user_sw", 2, Pins("L4"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("user_sw", 3, Pins("L5"), IOStandard("LVCMOS33"), Misc("PULLUP")),

    ("clk32", 0, Pins("J4"), IOStandard("LVCMOS33")),
    ("clk50", 0, Pins("K3"), IOStandard("LVCMOS33")),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("T3"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("R11"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("T10"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("P10"), IOStandard("LVCMOS33"))
    ),

    ("adc", 0,
        Subsignal("cs_n", Pins("F6"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("G6"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("H4"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("H5"), IOStandard("LVCMOS33"))
    ),

    ("serial", 0,
        Subsignal("tx", Pins("N6"), IOStandard("LVCMOS33")),  # FTDI D1
        Subsignal("rx", Pins("M7"), IOStandard("LVCMOS33"))   # FTDI D0
    ),

    ("audio", 0,
        Subsignal("a0", Pins("B8"), IOStandard("LVCMOS33")),
        Subsignal("a1", Pins("A8"), IOStandard("LVCMOS33"))
    ),

    ("sdram_clock", 0, Pins("G16"), IOStandard("LVCMOS33"), Misc("SLEW=FAST")),
    ("sdram", 0,
        Subsignal("a", Pins("T15 R16 P15 P16 N16 M15 M16 L16 K15 K16 R15 J16 H15")),
        Subsignal("dq", Pins("T13 T12 R12 T9 R9 T7 R7 T6 F16 E15 E16 D16 B16 B15 C16 C15")),
        Subsignal("we_n", Pins("R5")),
        Subsignal("ras_n", Pins("R2")),
        Subsignal("cas_n", Pins("T4")),
        Subsignal("cs_n", Pins("R1")),
        Subsignal("cke", Pins("H16")),
        Subsignal("ba", Pins("R14 T14")),
        Subsignal("dm", Pins("T5 F15")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")
    ),

    ("usb_fifo", 0,
        Subsignal("data", Pins("M7 N6 M6 P5 N5 P4 P2 P1")),
        Subsignal("rxf_n", Pins("N3")),
        Subsignal("txe_n", Pins("N1")),
        Subsignal("rd_n", Pins("M1")),
        Subsignal("wr_n", Pins("M2")),
        Subsignal("siwua", Pins("M3")),
        IOStandard("LVCMOS33"), Drive(8), Misc("SLEW=FAST")
    ),

    ("sd", 0,
        Subsignal("sck", Pins("L12")),
        Subsignal("d3", Pins("K12")),
        Subsignal("d", Pins("M10")),
        Subsignal("d1", Pins("L10")),
        Subsignal("d2", Pins("J11")),
        Subsignal("cmd", Pins("K11")),
        IOStandard("LVCMOS33")
    ),

    ("dvi_in", 0,
        Subsignal("clk_p", Pins("C9"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("A9"), IOStandard("TMDS_33")),
        Subsignal("data_p", Pins("C7 B6 B5"), IOStandard("TMDS_33")),
        Subsignal("data_n", Pins("A7 A6 A5"), IOStandard("TMDS_33")),
        Subsignal("scl", Pins("C1"), IOStandard("LVCMOS33")),
        Subsignal("sda", Pins("B1"), IOStandard("LVCMOS33"))
    ),

    ("dvi_out", 0,
        Subsignal("clk_p", Pins("B14"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("A14"), IOStandard("TMDS_33")),
        Subsignal("data_p", Pins("C13 B12 C11"), IOStandard("TMDS_33")),
        Subsignal("data_n", Pins("A13 A12 A11"), IOStandard("TMDS_33")),
    )
]

_connectors = [
    ("A", "E7 C8 D8 E8 D9 A10 B10 C10 E10 F9 F10 D11"),
    ("B", "E11 D14 D12 E12 E13 F13 F12 F14 G12 H14 J14"),
    ("C", "J13 J12 K14 L14 L13 M14 M13 N14 M12 N12 P12 M11"),
    ("D", "D6 C6 E6 C5"),
    ("E", "D5 A4 G5 A3 B3 A2 B2 C3 C2 D3 D1 E3"),
    ("F", "E2 E1 E4 F4 F5 G3 F3 G1 H3 H1 H2 J1")
]


class Platform(XilinxPlatform):
    default_clk_name = "clk32"
    default_clk_period = 31.25

    def __init__(self, device="xc6slx9", programmer="xc3sprog"):
        self.programmer = programmer
        XilinxPlatform.__init__(self, device+"-3-ftg256", _io, _connectors)

    def create_programmer(self):
        if self.programmer == "xc3sprog":
            return XC3SProg("minispartan6", "bscan_spi_minispartan6.bit")
        elif self.programmer == "fpgaprog":
            return FpgaProg()
        else:
            raise ValueError("{} programmer is not supported".format(programmer))
